@@ -67,6 +67,19 @@ MS2_PUBLIC RingStream *ring_start(MSFactory *factory, const char * file, int int
 MS2_PUBLIC RingStream *ring_start_with_cb(MSFactory *factory, const char * file, int interval, MSSndCard *sndcard, MSFilterNotifyFunc func, void * user_data);
 MS2_PUBLIC void ring_stop (RingStream * stream);
 
+/**
+ * Asks the ring filter to route to the selected sound card (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The RingStream object
+ * @param[in] sndcard_playback The wanted audio output soundcard
+ */
+MS2_PUBLIC void ring_stream_set_output_ms_snd_card(RingStream *stream, MSSndCard * sndcard_playback);
+
+/**
+ * Retrieve the current sound card from the audio playback filter (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The AudioStream object
+ */
+MS2_PUBLIC MSSndCard * ring_stream_get_output_ms_snd_card(RingStream *stream);
+
 /**
  * @}
 **/
@@ -81,7 +94,7 @@ typedef struct _MediaStream MediaStream;
 typedef void (*media_stream_process_rtcp_callback_t)(MediaStream *stream, mblk_t *m);
 
 struct _MSMediaStreamSessions{
-	RtpSession *rtp_session;
+    RtpSession *rtp_session;
 	MSSrtpCtx* srtp_context;
 	MSZrtpContext *zrtp_context;
 	MSDtlsSrtpContext *dtls_context;
@@ -145,7 +158,7 @@ struct _MediaStream {
 	MSFactory *factory;
 	MSBandwidthController *bandwidth_controller;
 	MSVideoQualityController *video_quality_controller;
-	MediaStreamDir direction;
+    MediaStreamDir direction;
 };
 
 MS2_PUBLIC void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStreamSessions *sessions);
@@ -290,6 +303,8 @@ MS2_PUBLIC void media_stream_iterate(MediaStream * stream);
 
 MS2_PUBLIC void media_stream_set_direction(MediaStream *stream, MediaStreamDir dir);
 
+MS2_PUBLIC void media_stream_reset_zrtp_context(MediaStream *stream);
+
 MS2_PUBLIC MediaStreamDir media_stream_get_direction(const MediaStream *stream);
 
 /**
@@ -323,7 +338,8 @@ typedef enum MSResourceType{
 	MSResourceFile,
 	MSResourceRtp,
 	MSResourceCamera,
-	MSResourceSoundcard
+	MSResourceSoundcard,
+	MSResourceVoid
 }MSResourceType;
 
 MS2_PUBLIC const char *ms_resource_type_to_string(MSResourceType type);
@@ -365,9 +381,13 @@ typedef struct _MSMediaStreamIO {
 
 MS2_PUBLIC bool_t ms_media_stream_io_is_consistent(const MSMediaStreamIO *io);
 
+typedef void (*AudioStreamIsSpeakingCallback)(void *user_pointer, uint32_t speaker_ssrc, bool_t is_speaking);
+typedef void (*MSAudioRouteChangedCallback)(void* audioStream, bool_t needReloadSoundDevices, char* newInputPort, char* newOutputPort);
 struct _AudioStream
 {
 	MediaStream ms;
+	MSSndCard *playcard;
+	MSSndCard *captcard;
 	MSFilter *soundread;
 	MSFilter *soundwrite;
 	MSFilter *dtmfgen;
@@ -424,6 +444,17 @@ struct _AudioStream
 	bool_t spk_eq_active;
 	bool_t use_ng;/*noise gate*/
 	bool_t is_ec_delay_set;
+	bool_t disable_record_on_mute;
+	float last_mic_gain_level_db;
+	struct _AudioStreamVolumes *participants_volumes;
+	int mixer_to_client_extension_id;
+	int client_to_mixer_extension_id;
+	uint32_t speaking_ssrc;
+	AudioStreamIsSpeakingCallback is_speaking_cb;
+	void *user_pointer;
+	bool_t is_speaking;
+	MSAudioRouteChangedCallback audio_route_changed_cb;
+	void * audio_route_changed_cb_user_data;
 };
 
 /**
@@ -431,7 +462,6 @@ struct _AudioStream
 **/
 typedef struct _AudioStream AudioStream;
 
-
 /* start a thread that does sampling->encoding->rtp_sending|rtp_receiving->decoding->playing */
 MS2_PUBLIC AudioStream *audio_stream_start(MSFactory* factory, RtpProfile * prof, int locport, const char *remip,
 				 int remport, int payload_type, int jitt_comp, bool_t echo_cancel);
@@ -622,6 +652,25 @@ static MS2_INLINE void audio_stream_enable_adaptive_bitrate_control(AudioStream
 static MS2_INLINE void audio_stream_enable_adaptive_jittcomp(AudioStream *stream, bool_t enabled) {
 	media_stream_enable_adaptive_jittcomp(&stream->ms, enabled);
 }
+/**
+ * Calling this method with disable=true will cause the microhone to be completely deactivated when muted
+ * Currently only implemented for IOS, this will cause the playback sound to be interrupted for a short moment while the audio is reconfigured.
+ * On IOS 14, it will also disable Apple's microphone recording indicator when microphone is muted.
+ *
+ * @param stream The stream.
+ * @param disable True if you wish to entirely stop the audio recording when muting the microphone.
+ */
+MS2_PUBLIC void audio_stream_disable_record_on_mute(AudioStream *stream, bool_t disable);
+
+/**
+ * Mute or unmute the microphone
+ * For IOS, if "audio_stream_disable_record_on_mute" was enabled, this will completely stop the microphone recording.
+ * Else, sound recording remains active but silence is sent instead of recorded audiow@
+ *
+ * @param stream The stream.
+ * @param enable Wether the microphone must be enabled.
+ */
+MS2_PUBLIC void audio_stream_enable_mic(AudioStream *stream, bool_t enable);
 
 /**
  * Apply a software gain on the microphone.
@@ -637,6 +686,7 @@ MS2_PUBLIC void audio_stream_set_mic_gain_db(AudioStream *stream, float gain_db)
 
 
 /**
+ * @deprecated
  * Like audio_stream_set_mic_gain_db() excepted that the gain is specified
  * in percentage.
  *
@@ -795,8 +845,53 @@ static MS2_INLINE RtpSession * audio_stream_get_rtp_session(const AudioStream *s
 	return media_stream_get_rtp_session(&stream->ms);
 }
 
+/**
+ * Sets the header extension id for mixer to client audio level indication.
+ * This has to be called before starting the audio stream.
+ *
+ * @param stream the audio stream
+ * @param extension_id the extension id
+ */
+MS2_PUBLIC void audio_stream_set_mixer_to_client_extension_id(AudioStream *stream, int extension_id);
 
+/**
+ * Sets the header extension id for client to mixer audio level indication.
+ * This has to be called before starting the audio stream.
+ *
+ * @param stream the audio stream
+ * @param extension_id the extension id
+ */
+MS2_PUBLIC void audio_stream_set_client_to_mixer_extension_id(AudioStream *stream, int extension_id);
 
+MS2_PUBLIC void audio_stream_set_is_speaking_callback(AudioStream *s, AudioStreamIsSpeakingCallback cb, void *user_pointer);
+/**
+ * Retrieve the volume of the given participant.
+ *
+ * @param stream the audio stream
+ * @param participant_ssrc the ssrc of the participant
+ * @return the volume of the participant in dbm0, if participant isn't found it will return the lowest volume.
+ */
+MS2_PUBLIC int audio_stream_get_participant_volume(const AudioStream *stream, uint32_t participant_ssrc);
+
+/* Map api to be able to keep participants volumes */
+typedef struct _AudioStreamVolumes AudioStreamVolumes;
+
+#define AUDIOSTREAMVOLUMES_NOT_FOUND INT16_MIN
+
+MS2_PUBLIC AudioStreamVolumes* audio_stream_volumes_new(void);
+MS2_PUBLIC void audio_stream_volumes_delete(AudioStreamVolumes *volumes);
+
+MS2_PUBLIC void audio_stream_volumes_insert(AudioStreamVolumes *volumes, uint32_t ssrc, int volume);
+MS2_PUBLIC void audio_stream_volumes_erase(AudioStreamVolumes *volumes, uint32_t ssrc);
+MS2_PUBLIC void audio_stream_volumes_clear(AudioStreamVolumes *volumes);
+
+MS2_PUBLIC int audio_stream_volumes_size(AudioStreamVolumes *volumes);
+MS2_PUBLIC int audio_stream_volumes_find(AudioStreamVolumes *volumes, uint32_t ssrc);
+
+MS2_PUBLIC void audio_stream_volumes_reset_values(AudioStreamVolumes *volumes);
+
+bool_t audio_stream_volumes_is_speaking(AudioStreamVolumes *volumes);
+uint32_t audio_stream_volumes_get_best(AudioStreamVolumes *volumes);
 /**
  * @}
 **/
@@ -810,6 +905,7 @@ static MS2_INLINE RtpSession * audio_stream_get_rtp_session(const AudioStream *s
 typedef void (*VideoStreamRenderCallback)(void *user_pointer, const MSPicture *local_view, const MSPicture *remote_view);
 typedef void (*VideoStreamEventCallback)(void *user_pointer, const MSFilter *f, const unsigned int event_id, const void *args);
 typedef void (*VideoStreamCameraNotWorkingCallback)(void *user_pointer, const MSWebCam *old_webcam);
+typedef void (*VideoStreamEncoderControlCb)(struct _VideoStream *, unsigned int method_id, void *arg, void *user_data);
 
 struct _MediastreamVideoStat
 {
@@ -866,6 +962,8 @@ struct _VideoStream
 	VideoStreamCameraNotWorkingCallback cameracb;
 	void *camera_pointer;
 	MediaStreamVideoStat ms_video_stat;
+	VideoStreamEncoderControlCb encoder_control_cb;
+	void *encoder_control_cb_user_data;
 	bool_t use_preview_window;
 	bool_t enable_qrcode_decoder;
 	bool_t freeze_on_error;
@@ -1006,6 +1104,12 @@ MS2_PUBLIC void video_stream_update_video_params(VideoStream *stream);
 /*function to call periodically to handle various events */
 MS2_PUBLIC void video_stream_iterate(VideoStream *stream);
 
+/*
+ * Assign a specific callback to process PLI, SLI, FIR received by RTCP.
+ * When set to NULL, or not assigned, the default behavior is to target these commands to the video encoder.
+ */ 
+MS2_PUBLIC void video_stream_set_encoder_control_callback(VideoStream *stream, VideoStreamEncoderControlCb cb, void *user_data);
+
 /**
  * Asks the video stream to send a Full-Intra Request.
  * @param[in] stream The videostream object.
@@ -1208,6 +1312,8 @@ MS2_PUBLIC void video_stream_close_remote_record(VideoStream *stream);
 MS2_PUBLIC void video_stream_enable_retransmission_on_nack(VideoStream *stream, bool_t enable);
 MS2_PUBLIC void video_stream_set_retransmission_on_nack_max_packet(VideoStream *stream, unsigned int max);
 
+MS2_PUBLIC void video_stream_enable_fec(VideoStream *stream, char* local_ip, int local_port, int local_rtcp_port, char* remote_ip, int remote_port, int L, int D);
+
 /**
  * Small API to display a local preview window.
 **/
@@ -1262,6 +1368,41 @@ MS2_PUBLIC RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int l
  */
 MS2_PUBLIC void audio_stream_set_audio_route(AudioStream *stream, MSAudioRoute route);
 
+/**
+ *  Set a callback to be called when an audio route change is notified (IOS only)
+ * @param[in] stream The AudioStream object
+ * @param[in] callback The function to call when receiving the route change event
+ * @param[in] audio_route_changed_cb_user_data Context for the callback
+ */
+MS2_PUBLIC void audio_stream_set_audio_route_changed_callback(AudioStream *stream, MSAudioRouteChangedCallback callback, void *audio_route_changed_cb_user_data);
+/**
+ * Asks the audio capture filter to route to the selected sound card (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The AudioStream object
+ * @param[in] sndcard_capture The wanted audio input soundcard
+ * @return '0' if MS_AUDIO_CAPTURE_SET_INTERNAL_ID has been successfully called, '-1' if not
+ */
+MS2_PUBLIC int audio_stream_set_input_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_capture);
+
+/**
+ * Asks the audio playback filter to route to the selected sound card (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The AudioStream object
+ * @param[in] sndcard_playback The wanted audio output soundcard
+ * @return '0' if MS_AUDIO_PLAYBACK_SET_INTERNAL_ID has been successfully called, '-1' if not
+ */
+MS2_PUBLIC int audio_stream_set_output_ms_snd_card(AudioStream *stream, MSSndCard * sndcard_playback);
+
+/**
+ * Retrieve the current sound card from the audio capture filter (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The AudioStream object
+ */
+MS2_PUBLIC MSSndCard * audio_stream_get_input_ms_snd_card(AudioStream *stream);
+
+/**
+ * Retrieve the current sound card from the audio playback filter (currently only used for AAudio and OpenSLES)
+ * @param[in] stream The AudioStream object
+ */
+MS2_PUBLIC MSSndCard * audio_stream_get_output_ms_snd_card(AudioStream *stream);
+
 /**
  * @}
 **/
