@@ -22,6 +22,7 @@
 #include <mediastreamer2/msticker.h>
 #include <mediastreamer2/mssndcard.h>
 #include <mediastreamer2/devices.h>
+#include <mediastreamer2/android_utils.h>
 
 #include <sys/types.h>
 #include <string.h>
@@ -96,15 +97,21 @@ static int DeviceFavoriteBufferSize = 256;
 
 using namespace fake_opensles;
 
-static MSSndCard *android_snd_card_new(MSSndCardManager *m);
+static void android_snd_card_device_create(JNIEnv *env, jobject deviceInfo, SoundDeviceDescription *deviceDescription, MSSndCardManager *m);
+static void snd_card_device_create(int device_id, const char * name, MSSndCardDeviceType type, unsigned int capabilities, SoundDeviceDescription *deviceDescription, MSSndCardManager *m);
 static MSFilter *ms_android_snd_read_new(MSFactory *factory);
 static MSFilter *ms_android_snd_write_new(MSFactory* factory);
 
 struct OpenSLESContext {
 	OpenSLESContext() {
+
 		samplerate = DeviceFavoriteSampleRate;
 		nchannels = 1;
 		builtin_aec = false;
+
+		engineObject = NULL;
+		engineEngine = NULL;
+
 	}
 
 	int samplerate;
@@ -127,6 +134,13 @@ struct OpenSLESOutputContext {
 		currentBuffer = 0;
 		playBuffer[0] = NULL;
 		playBuffer[1] = NULL;
+
+		// Invalidate objects
+		outputMixObject = NULL;
+		playerObject = NULL;
+		playerPlay = NULL;
+		playerBufferQueue = NULL;
+		playerConfig = NULL;
 	}
 
 	~OpenSLESOutputContext() {
@@ -148,11 +162,16 @@ struct OpenSLESOutputContext {
 		MSSndCardStreamType type = ms_snd_card_get_stream_type(soundCard);
 		streamType = SL_ANDROID_STREAM_VOICE;
 		if (type == MS_SND_CARD_STREAM_RING) {
+			ms_message("[OpenSLES] Sound card configured for RING stream");
 			streamType = SL_ANDROID_STREAM_RING;
 		} else if (type == MS_SND_CARD_STREAM_MEDIA) {
+			ms_message("[OpenSLES] Sound card configured for MEDIA stream");
 			streamType = SL_ANDROID_STREAM_MEDIA;
 		} else if (type == MS_SND_CARD_STREAM_DTMF) {
+			ms_message("[OpenSLES] Sound card configured for DTMF stream");
 			streamType = SL_ANDROID_STREAM_NOTIFICATION;
+		} else {
+			ms_message("[OpenSLES] Sound card configured for VOICE stream");
 		}
 	}
 
@@ -193,6 +212,8 @@ struct OpenSLESInputContext {
 		currentBuffer = 0;
 		recBuffer[0] = NULL;
 		recBuffer[1] = NULL;
+		voiceRecognitionMode = false;
+		deviceChanged = false;
 	}
 
 	~OpenSLESInputContext() {
@@ -217,6 +238,7 @@ struct OpenSLESInputContext {
 	queue_t q;
 	ms_mutex_t mutex;
 	MSTickerSynchronizer *mTickerSynchronizer;
+	MSSndCard *soundCard;
 	MSFilter *mFilter;
 	int64_t read_samples;
 	jobject aec;
@@ -225,6 +247,8 @@ struct OpenSLESInputContext {
 	int inBufSize;
 	int currentBuffer;
 	double mAvSkew;
+	bool voiceRecognitionMode;
+	bool deviceChanged;
 };
 
 static SLuint32 convertSamplerate(int samplerate)
@@ -233,8 +257,7 @@ static SLuint32 convertSamplerate(int samplerate)
     case 8000:
         return SL_SAMPLINGRATE_8;
         break;
-    case 16000:
-        return SL_SAMPLINGRATE_16;
+    case 16000: return SL_SAMPLINGRATE_16;
         break;
     case 22050:
         return SL_SAMPLINGRATE_22_05;
@@ -253,16 +276,46 @@ static SLuint32 convertSamplerate(int samplerate)
     }
 }
 
+static void android_snd_card_add_devices(MSDevicesInfo *devices, SoundDeviceDescription* deviceDescription, MSSndCardManager *m) {
+	JNIEnv *env = ms_get_jni_env();
+	int sdkVersion = ms_android_get_sdk_version(env);
+
+	//GetDevices is only available from API23
+	if (sdkVersion >= 23) {
+		// Get all devices
+		jobject devices = ms_android_get_all_devices(env, "all");
+
+		// extract required information from every device
+		jobjectArray deviceArray = reinterpret_cast<jobjectArray>(devices);
+		jsize deviceNumber = (int) env->GetArrayLength(deviceArray);
+		ms_message("[OpenSLES] Create soundcards for %0d devices", deviceNumber);
+
+		for (int idx=0; idx < deviceNumber; idx++) {
+			jobject deviceInfo = env->GetObjectArrayElement(deviceArray, idx);
+			android_snd_card_device_create(env, deviceInfo, deviceDescription, m);
+		}
+	} else {
+		//For devices running API older than API23, only 3 devices are created: microphone, speaker and earpiece
+		snd_card_device_create(-1, "Microphone", MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_MICROPHONE, MS_SND_CARD_CAP_CAPTURE, deviceDescription, m);
+		snd_card_device_create(-1, "Earpiece", MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE, MS_SND_CARD_CAP_PLAYBACK, deviceDescription, m);
+		snd_card_device_create(-1, "Speaker", MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_SPEAKER, MS_SND_CARD_CAP_PLAYBACK, deviceDescription, m);
+	}
+
+}
+
 static void android_snd_card_detect(MSSndCardManager *m) {
-	SoundDeviceDescription* d = NULL;
+	SoundDeviceDescription* deviceDescription = NULL;
 	MSDevicesInfo *devices = NULL;
+
 	if (initOpenSLES() == 0) { // Try to dlopen libOpenSLES
 		ms_message("[OpenSLES] libOpenSLES correctly loaded, creating OpenSLES MS soundcard");
 		devices = ms_factory_get_devices_info(m->factory);
-		d = ms_devices_info_get_sound_device_description(devices);
-		if (d->flags & DEVICE_HAS_CRAPPY_OPENSLES) return;
-		MSSndCard *card = android_snd_card_new(m);
-		ms_snd_card_manager_add_card(m, card);
+		deviceDescription = ms_devices_info_get_sound_device_description(devices);
+		if (deviceDescription->flags & DEVICE_HAS_CRAPPY_OPENSLES) return;
+		
+		DeviceFavoriteSampleRate = ms_android_get_preferred_sample_rate();
+		DeviceFavoriteBufferSize = ms_android_get_preferred_buffer_size();
+		android_snd_card_add_devices(devices, deviceDescription, m);
 	} else {
 		ms_warning("[OpenSLES] Failed to dlopen libOpenSLES, OpenSLES MS soundcard unavailable");
 	}
@@ -271,6 +324,7 @@ static void android_snd_card_detect(MSSndCardManager *m) {
 static SLresult opensles_engine_init(OpenSLESContext *ctx) {
 	SLresult result;
 
+
 	result = slwCreateEngine(&(ctx->engineObject), 0, NULL, 0, NULL, NULL);
 	if (result != SL_RESULT_SUCCESS) {
 		ms_error("[OpenSLES] Error %u while creating SL engine", result);
@@ -293,26 +347,36 @@ static SLresult opensles_engine_init(OpenSLESContext *ctx) {
 	return result;
 }
 
-static OpenSLESContext* opensles_context_init() {
-	OpenSLESContext* ctx = new OpenSLESContext();
-	opensles_engine_init(ctx);
-	return ctx;
-}
-
 static void android_native_snd_card_init(MSSndCard *card) {
-
+	OpenSLESContext* context = new OpenSLESContext();
+	opensles_engine_init(context);
+	card->data = context;
 }
 
 static void android_native_snd_card_uninit(MSSndCard *card) {
 	OpenSLESContext *ctx = (OpenSLESContext*)card->data;
-	ms_warning("[OpenSLES] Deletion of OpenSLES context [%p]", ctx);
+	ms_message("[OpenSLES] Deleting card [%p]: name [%s] device ID [%0d] type [%s]", card, card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type));
+	
 	if (ctx->engineObject != NULL) {
-                (*ctx->engineObject)->Destroy(ctx->engineObject);
-                ctx->engineObject = NULL;
-                ctx->engineEngine = NULL;
-        }
+		(*ctx->engineObject)->Destroy(ctx->engineObject);
+		ctx->engineObject = NULL;
+		ctx->engineEngine = NULL;
+	}
+
+	ms_message("[OpenSLES] Deletion of OpenSLES context [%p]", ctx);
+	delete ctx;
 }
 
+static void opensles_reset_recorder(OpenSLESInputContext *ictx) {
+	if (ictx->recorderObject != NULL) {
+		(*ictx->recorderObject)->Destroy(ictx->recorderObject);
+		ictx->recorderObject = NULL;
+		ictx->recorderRecord = NULL;
+		ictx->recorderBufferQueue = NULL;
+	}
+}
+
+
 static SLresult opensles_recorder_init(OpenSLESInputContext *ictx) {
 	SLresult result;
 	SLuint32 sample_rate = convertSamplerate(ictx->opensles_context->samplerate);
@@ -362,36 +426,47 @@ static SLresult opensles_recorder_init(OpenSLESInputContext *ictx) {
 
 	result = (*ictx->opensles_context->engineEngine)->CreateAudioRecorder(ictx->opensles_context->engineEngine, &ictx->recorderObject, &audio_src, &audio_sink, 2, ids, req);
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while creating the audio recorder", result);
 		return result;
 	}
 
 	result = (*ictx->recorderObject)->GetInterface(ictx->recorderObject, SLW_IID_ANDROIDCONFIGURATION, &ictx->recorderConfig);
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while getting the recorder's android config interface", result);
 		return result;
 	}
 
+	if (ictx->voiceRecognitionMode) {
+		// Voice recognition preset works better when recording voice message
+		ms_message("[OpenSLES] Using preset voice recognition (%i) instead of %i", SL_ANDROID_RECORDING_PRESET_VOICE_RECOGNITION, ictx->streamType);
+		ictx->streamType = SL_ANDROID_RECORDING_PRESET_VOICE_RECOGNITION;
+	}
 	result = (*ictx->recorderConfig)->SetConfiguration(ictx->recorderConfig, SL_ANDROID_KEY_RECORDING_PRESET, &ictx->streamType, sizeof(SLint32));
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while setting the audio recorder configuration", result);
 		return result;
 	}
 
 	result = (*ictx->recorderObject)->Realize(ictx->recorderObject, SL_BOOLEAN_FALSE);
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while realizing the audio recorder", result);
 		return result;
 	}
 
 	result = (*ictx->recorderObject)->GetInterface(ictx->recorderObject, SLW_IID_RECORD, &ictx->recorderRecord);
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while getting the audio recorder's interface", result);
 		return result;
 	}
 
 	result = (*ictx->recorderObject)->GetInterface(ictx->recorderObject, SLW_IID_ANDROIDSIMPLEBUFFERQUEUE, &ictx->recorderBufferQueue);
 	if (SL_RESULT_SUCCESS != result) {
+		opensles_reset_recorder(ictx);
 		ms_error("[OpenSLES] Error %u while getting the audio recorder's buffer interface", result);
 		return result;
 	}
@@ -409,22 +484,6 @@ static SLresult opensles_recorder_init(OpenSLESInputContext *ictx) {
 static void opensles_recorder_callback(SLAndroidSimpleBufferQueueItf bq, void *context) {
 	SLresult result;
 	OpenSLESInputContext *ictx = (OpenSLESInputContext *)context;
-
-	if (ictx->mTickerSynchronizer == NULL) {
-		MSFilter *obj = ictx->mFilter;
-		/*
-		 * ABSOLUTE HORRIBLE HACK. We temporarily disable logs to prevent ms_ticker_set_synchronizer() to output a debug log.
-		 * This is horrible because this also suspends logs for all concurrent threads during these two lines of code.
-		 * Possible way to do better:
-		 *  1) understand why AudioRecord thread doesn't detach.
-		 *  2) disable logs just for this thread (using a TLS)
-		 */
-		int loglevel=bctbx_get_log_level_mask(BCTBX_LOG_DOMAIN);
-		bctbx_set_log_level_mask(BCTBX_LOG_DOMAIN, BCTBX_LOG_ERROR|BCTBX_LOG_FATAL);
-		ictx->mTickerSynchronizer = ms_ticker_synchronizer_new();
-		ms_ticker_set_synchronizer(obj->ticker, ictx->mTickerSynchronizer);
-		bctbx_set_log_level_mask(BCTBX_LOG_DOMAIN, loglevel);
-	}
 	ictx->read_samples += ictx->inBufSize / sizeof(int16_t);
 
 	mblk_t *m = allocb(ictx->inBufSize, 0);
@@ -432,7 +491,6 @@ static void opensles_recorder_callback(SLAndroidSimpleBufferQueueItf bq, void *c
 	m->b_wptr += ictx->inBufSize;
 
 	ms_mutex_lock(&ictx->mutex);
-	ictx->mAvSkew = ms_ticker_synchronizer_update(ictx->mTickerSynchronizer, ictx->read_samples, (unsigned int)ictx->opensles_context->samplerate);
 	putq(&ictx->q, m);
 	ms_mutex_unlock(&ictx->mutex);
 
@@ -502,6 +560,11 @@ static void android_snd_read_preprocess(MSFilter *obj) {
 	ictx->inBufSize = DeviceFavoriteBufferSize * sizeof(int16_t) * ictx->opensles_context->nchannels;
 	ictx->recBuffer[0] = (uint8_t *) calloc(ictx->inBufSize, sizeof(uint8_t));
 	ictx->recBuffer[1] = (uint8_t *) calloc(ictx->inBufSize, sizeof(uint8_t));
+	
+	if (ictx->mTickerSynchronizer == NULL) {
+		ictx->mTickerSynchronizer = ms_ticker_synchronizer_new();
+		ms_ticker_set_synchronizer(obj->ticker, ictx->mTickerSynchronizer);
+	}
 
 	if (SL_RESULT_SUCCESS != opensles_recorder_init(ictx)) {
 	    ms_error("[OpenSLES] Problem when initialization of opensles recorder");
@@ -520,25 +583,33 @@ static void android_snd_read_preprocess(MSFilter *obj) {
 static void android_snd_read_process(MSFilter *obj) {
 	OpenSLESInputContext *ictx = (OpenSLESInputContext*) obj->data;
 	mblk_t *m;
+	if (ictx->deviceChanged &&  ictx->mTickerSynchronizer){
+		ms_mutex_lock(&ictx->mutex);
+		ms_ticker_synchronizer_resync(ictx->mTickerSynchronizer);
+		ms_message("[OpenSLES] resync ticket synchronizer to avoid audio delay");
+		ictx->deviceChanged = false;
+		ms_mutex_unlock(&ictx->mutex);
+	}
 
 	if (obj->ticker->time % 1000 == 0) {
-	    if (ictx->recorderBufferQueue == NULL) {
-	        ms_message("[OpenSLES] Trying to init opensles recorder on process");
-	        if (SL_RESULT_SUCCESS != opensles_recorder_init(ictx)) {
-                ms_error("[OpenSLES] Problem when initialization of opensles recorder");
-            } else if (SL_RESULT_SUCCESS != opensles_recorder_callback_init(ictx)) {
-            	ms_error("[OpenSLES] Problem when initialization of opensles recorder callback");
-            }
-	    }
+		if (ictx->recorderBufferQueue == NULL) {
+			ms_message("[OpenSLES] Trying to init opensles recorder on process");
+			if (SL_RESULT_SUCCESS != opensles_recorder_init(ictx)) {
+				ms_error("[OpenSLES] Problem when initialization of opensles recorder");
+			} else if (SL_RESULT_SUCCESS != opensles_recorder_callback_init(ictx)) {
+				ms_error("[OpenSLES] Problem when initialization of opensles recorder callback");
+			}
+		}
 	}
 
 	ms_mutex_lock(&ictx->mutex);
 	while ((m = getq(&ictx->q)) != NULL) {
 		ms_queue_put(obj->outputs[0], m);
 	}
+	ictx->mAvSkew = ms_ticker_synchronizer_update(ictx->mTickerSynchronizer, ictx->read_samples, (unsigned int)ictx->opensles_context->samplerate);
 	ms_mutex_unlock(&ictx->mutex);
 	if (obj->ticker->time % 5000 == 0)
-			ms_message("[OpenSLES] sound/wall clock skew is average=%g ms", ictx->mAvSkew);
+		ms_message("[OpenSLES] sound/wall clock skew is average=%g ms", ictx->mAvSkew);
 
 }
 
@@ -575,13 +646,21 @@ static void android_snd_read_postprocess(MSFilter *obj) {
 
 	ms_ticker_set_synchronizer(obj->ticker, NULL);
 	ms_mutex_lock(&ictx->mutex);
-	ms_ticker_synchronizer_destroy(ictx->mTickerSynchronizer);
-	ictx->mTickerSynchronizer = NULL;
+	if (ictx->mTickerSynchronizer) {
+		ms_ticker_synchronizer_destroy(ictx->mTickerSynchronizer);
+		ictx->mTickerSynchronizer = NULL;
+	}
 	ms_mutex_unlock(&ictx->mutex);
 }
 
 static void android_snd_read_uninit(MSFilter *obj) {
 	OpenSLESInputContext *ictx = (OpenSLESInputContext*)obj->data;
+
+	if (ictx->soundCard) {
+		ms_snd_card_unref(ictx->soundCard);
+		ictx->soundCard = NULL;
+	}
+	
 	delete ictx;
 }
 
@@ -620,7 +699,41 @@ static int android_snd_read_get_nchannels(MSFilter *obj, void *data) {
 	return 0;
 }
 
-static int android_snd_read_hack_speaker_state(MSFilter *f, void *arg) {
+static int android_snd_read_configure_soundcard(MSFilter *obj, void *data) {
+	MSSndCard *card = (MSSndCard*)data;
+	OpenSLESInputContext *ictx = (OpenSLESInputContext*)obj->data;
+	// Check if device_id is different and/or device_type is different
+	// For API < 23, all device ID are identical but the device type is different
+	if ((ictx->soundCard->internal_id != card->internal_id) || (ictx->soundCard->device_type != card->device_type)) {
+		ms_mutex_lock(&ictx->mutex);
+		if (ictx->soundCard) {
+			ms_snd_card_unref(ictx->soundCard);
+			ictx->soundCard = NULL;
+		}
+		ictx->soundCard = ms_snd_card_ref(card);
+		OpenSLESContext* opensles_context = (OpenSLESContext*)card->data;
+		ictx->setContext(opensles_context);
+		ictx->deviceChanged = true;
+		ms_mutex_unlock(&ictx->mutex);
+	}
+	return 0;
+}
+
+static int android_snd_read_get_device_id(MSFilter *obj, void *data) {
+	int *n = (int*)data;
+	OpenSLESInputContext *ictx = (OpenSLESInputContext*)obj->data;
+	*n = ictx->soundCard->internal_id;
+	return 0;
+}
+
+static int android_snd_read_hack_speaker_state(MSFilter *obj, void *data) {
+	return 0;
+}
+
+static int android_snd_read_enable_voice_rec(MSFilter *obj, void *data) {
+	bool *enabled = (bool*)data;
+	OpenSLESInputContext *ictx = (OpenSLESInputContext*)obj->data;
+	ictx->voiceRecognitionMode = !!(*enabled);
 	return 0;
 }
 
@@ -630,6 +743,9 @@ static MSFilterMethod android_snd_read_methods[] = {
 	{MS_FILTER_SET_NCHANNELS, android_snd_read_set_nchannels},
 	{MS_FILTER_GET_NCHANNELS, android_snd_read_get_nchannels},
 	{MS_AUDIO_CAPTURE_FORCE_SPEAKER_STATE, android_snd_read_hack_speaker_state},
+	{MS_AUDIO_CAPTURE_SET_INTERNAL_ID, android_snd_read_configure_soundcard},
+	{MS_AUDIO_CAPTURE_GET_INTERNAL_ID, android_snd_read_get_device_id},
+	{MS_AUDIO_CAPTURE_ENABLE_VOICE_REC, android_snd_read_enable_voice_rec},
 	{0,NULL}
 };
 
@@ -657,6 +773,7 @@ static MSFilter* ms_android_snd_read_new(MSFactory *factory) {
 static MSFilter *android_snd_card_create_reader(MSSndCard *card) {
 	MSFilter *f = ms_android_snd_read_new(ms_snd_card_get_factory(card));
 	OpenSLESInputContext *ictx = static_cast<OpenSLESInputContext*>(f->data);
+	ictx->soundCard = ms_snd_card_ref(card);
 	ictx->setContext((OpenSLESContext*)card->data);
 	return f;
 }
@@ -668,12 +785,16 @@ static SLresult opensles_mixer_init(OpenSLESOutputContext *octx) {
 	const SLboolean req[] = {};
 	result = (*octx->opensles_context->engineEngine)->CreateOutputMix(octx->opensles_context->engineEngine, &(octx->outputMixObject), nbInterface, ids, req);
 	if (result != SL_RESULT_SUCCESS) {
+
 		ms_error("[OpenSLES] Error %u while creating output mixer", result);
 		return result;
 	}
 
 	result = (*octx->outputMixObject)->Realize(octx->outputMixObject, SL_BOOLEAN_FALSE);
 	if (result != SL_RESULT_SUCCESS) {
+		if (octx->outputMixObject != NULL) {
+			octx->outputMixObject = NULL;
+		}
 		ms_error("[OpenSLES] Error %u while realizing output mixer", result);
 		return result;
 	}
@@ -681,6 +802,15 @@ static SLresult opensles_mixer_init(OpenSLESOutputContext *octx) {
 	return result;
 }
 
+static void opensles_reset_sink(OpenSLESOutputContext *octx) {
+	if (octx->playerObject != NULL) {
+		(*octx->playerObject)->Destroy(octx->playerObject);
+		octx->playerObject = NULL;
+		octx->playerPlay = NULL;
+		octx->playerBufferQueue = NULL;
+	}
+}
+
 static SLresult opensles_sink_init(OpenSLESOutputContext *octx) {
 	SLresult result;
 	SLuint32 sample_rate = convertSamplerate(octx->opensles_context->samplerate);
@@ -726,12 +856,14 @@ static SLresult opensles_sink_init(OpenSLESOutputContext *octx) {
 	const SLboolean req[] = { SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE, SL_BOOLEAN_TRUE};
 	result = (*octx->opensles_context->engineEngine)->CreateAudioPlayer(octx->opensles_context->engineEngine, &(octx->playerObject), &audio_src, &audio_sink, nbInterface, ids, req);
 	if (result != SL_RESULT_SUCCESS) {
+		opensles_reset_sink(octx);
 		ms_error("[OpenSLES] Error %u while creating ouput audio player", result);
 		return result;
 	}
 
 	result = (*octx->playerObject)->GetInterface(octx->playerObject, SLW_IID_ANDROIDCONFIGURATION, &octx->playerConfig);
 	if (result != SL_RESULT_SUCCESS) {
+		opensles_reset_sink(octx);
 		ms_error("[OpenSLES] Error %u while getting android configuration interface", result);
 		return result;
 	}
@@ -739,18 +871,21 @@ static SLresult opensles_sink_init(OpenSLESOutputContext *octx) {
 	octx->updateStreamTypeFromMsSndCard();
 	result = (*octx->playerConfig)->SetConfiguration(octx->playerConfig, SL_ANDROID_KEY_STREAM_TYPE, &octx->streamType, sizeof(SLint32));
 	if (result != SL_RESULT_SUCCESS) {
+		opensles_reset_sink(octx);
 		ms_error("[OpenSLES] Error %u while setting stream type configuration", result);
 		return result;
 	}
 
 	result = (*octx->playerObject)->Realize(octx->playerObject, SL_BOOLEAN_FALSE);
 	if (result != SL_RESULT_SUCCESS) {
+		opensles_reset_sink(octx);
 		ms_error("[OpenSLES] Error %u while realizing output sink", result);
 		return result;
 	}
 
 	result = (*octx->playerObject)->GetInterface(octx->playerObject, SLW_IID_PLAY, &(octx->playerPlay));
 	if (result != SL_RESULT_SUCCESS) {
+		opensles_reset_sink(octx);
 		ms_error("[OpenSLES] Error %u while getting output sink interface play", result);
 		return result;
 	}
@@ -840,7 +975,7 @@ static SLresult opensles_player_callback_init(OpenSLESOutputContext *octx) {
 static MSFilter *android_snd_card_create_writer(MSSndCard *card) {
 	MSFilter *f = ms_android_snd_write_new(ms_snd_card_get_factory(card));
 	OpenSLESOutputContext *octx = static_cast<OpenSLESOutputContext*>(f->data);
-	octx->soundCard = card;
+	octx->soundCard = ms_snd_card_ref(card);
 	octx->setContext((OpenSLESContext*)card->data);
 	return f;
 }
@@ -852,6 +987,12 @@ static void android_snd_write_init(MSFilter *obj){
 
 static void android_snd_write_uninit(MSFilter *obj){
 	OpenSLESOutputContext *octx = (OpenSLESOutputContext*)obj->data;
+
+	if (octx->soundCard) {
+		ms_snd_card_unref(octx->soundCard);
+		octx->soundCard = NULL;
+	}
+
 	delete octx;
 }
 
@@ -891,6 +1032,34 @@ static int android_snd_write_get_nchannels(MSFilter *obj, void *data) {
 	return 0;
 }
 
+static int android_snd_write_configure_soundcard(MSFilter *obj, void *data) {
+	MSSndCard *card = (MSSndCard*)data;
+	OpenSLESOutputContext *octx = (OpenSLESOutputContext*)obj->data;
+
+	// Check if device_id is different and/or device_type is different
+	// For API < 23, all device ID are identical but the device type is different
+	if ((octx->soundCard->internal_id != card->internal_id) || (octx->soundCard->device_type != card->device_type)) {
+		ms_mutex_lock(&octx->mutex);
+		if (octx->soundCard) {
+			ms_snd_card_unref(octx->soundCard);
+			octx->soundCard = NULL;
+		}
+		octx->soundCard = ms_snd_card_ref(card);
+		octx->setContext((OpenSLESContext*)card->data);
+		JNIEnv *env = ms_get_jni_env();
+		ms_android_change_device(env, card->device_type);
+		ms_mutex_unlock(&octx->mutex);
+	}
+	return 0;
+}
+
+static int android_snd_write_get_device_id(MSFilter *obj, void *data) {
+	int *n = (int*)data;
+	OpenSLESOutputContext *octx = (OpenSLESOutputContext*)obj->data;
+	*n = octx->soundCard->internal_id;
+	return 0;
+}
+
 static void android_snd_write_preprocess(MSFilter *obj) {
 	OpenSLESOutputContext *octx = (OpenSLESOutputContext*)obj->data;
 	SLresult result;
@@ -916,11 +1085,16 @@ static void android_snd_write_preprocess(MSFilter *obj) {
 	}
 
 	octx->nbufs = 0;
+
+	// Ensure consistency between the soundcard the core is setting and the one actually used as an output
+	JNIEnv *env = ms_get_jni_env();
+	ms_android_change_device(env, octx->soundCard->device_type);
 }
 
 static void android_snd_write_process(MSFilter *obj) {
 	OpenSLESOutputContext *octx = (OpenSLESOutputContext*)obj->data;
 	ms_mutex_lock(&octx->mutex);
+
 	ms_flow_controlled_bufferizer_put_from_queue(&octx->buffer, obj->inputs[0]);
 	ms_mutex_unlock(&octx->mutex);
 }
@@ -954,6 +1128,10 @@ static void android_snd_write_postprocess(MSFilter *obj) {
 		(*octx->outputMixObject)->Destroy(octx->outputMixObject);
 		octx->outputMixObject = NULL;
 	}
+
+	// At the end of a call, postprocess is called therefore here the output device can be changed to earpiece in the audio manager
+	JNIEnv *env = ms_get_jni_env();
+	ms_android_change_device(env, MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE);
 	free(octx->playBuffer[0]);
 	octx->playBuffer[0]=NULL;
 	free(octx->playBuffer[1]);
@@ -965,6 +1143,8 @@ static MSFilterMethod android_snd_write_methods[] = {
 	{MS_FILTER_GET_SAMPLE_RATE, android_snd_write_get_sample_rate},
 	{MS_FILTER_SET_NCHANNELS, android_snd_write_set_nchannels},
 	{MS_FILTER_GET_NCHANNELS, android_snd_write_get_nchannels},
+	{MS_AUDIO_PLAYBACK_SET_INTERNAL_ID, android_snd_write_configure_soundcard},
+	{MS_AUDIO_PLAYBACK_GET_INTERNAL_ID, android_snd_write_get_device_id},
 	{0,NULL}
 };
 
@@ -1003,48 +1183,92 @@ MSSndCardDesc android_native_snd_opensles_card_desc = {
 	android_native_snd_card_uninit
 };
 
-static MSSndCard* android_snd_card_new(MSSndCardManager *m) {
-	MSSndCard* card = NULL;
-	SoundDeviceDescription *d = NULL;
-	MSDevicesInfo *devices = NULL;
+static void snd_card_device_create_extra_fields(MSSndCardManager *m, MSSndCard *card, SoundDeviceDescription *deviceDescription) {
+	OpenSLESContext *card_data = (OpenSLESContext*)card->data;
+	
+	if ((card->capabilities & MS_SND_CARD_CAP_CAPTURE) == MS_SND_CARD_CAP_CAPTURE) {
+		bool hasBuiltingOpenslesAec = (deviceDescription->flags & DEVICE_HAS_BUILTIN_OPENSLES_AEC) == DEVICE_HAS_BUILTIN_OPENSLES_AEC;
+		bool hasBuiltinAec = (deviceDescription->flags & DEVICE_HAS_BUILTIN_AEC) == DEVICE_HAS_BUILTIN_AEC;
+		bool hasCrappyAec = (deviceDescription->flags & DEVICE_HAS_BUILTIN_AEC_CRAPPY) == DEVICE_HAS_BUILTIN_AEC_CRAPPY;
 
-	card = ms_snd_card_new(&android_native_snd_opensles_card_desc);
-	card->name = ms_strdup("android sound card");
-
-	devices = ms_factory_get_devices_info(m->factory);
-	d = ms_devices_info_get_sound_device_description(devices);
-
-	JNIEnv *env = ms_get_jni_env();
-	jclass mediastreamerAndroidContextClass = env->FindClass("org/linphone/mediastream/MediastreamerAndroidContext");
-	if (mediastreamerAndroidContextClass != NULL) {
-		jmethodID getBufferSize = env->GetStaticMethodID(mediastreamerAndroidContextClass, "getDeviceFavoriteBufferSize", "()I");
-		if (getBufferSize != NULL) {
-				jint ret = env->CallStaticIntMethod(mediastreamerAndroidContextClass, getBufferSize);
-				DeviceFavoriteBufferSize = (int)ret;
-				ms_message("[OpenSLES] Using %i for buffer size value", DeviceFavoriteBufferSize);
+		if (hasBuiltingOpenslesAec) {
+			ms_message("[OpenSLES] Adding MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER flag to soundcard ([%s] device ID [%0d] type [%s]) because of DEVICE_HAS_BUILTIN_OPENSLES_AEC", card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type));
+			card->capabilities |= MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
+			card_data->builtin_aec = true;
+		} else if (hasBuiltinAec && !hasCrappyAec && card->device_type == MS_SND_CARD_DEVICE_TYPE_MICROPHONE) {
+			// Only apply this workaround to default microphone, not any other device that has CAPTURE capability
+			ms_warning("[OpenSLES] Removing MS_SND_CARD_CAP_CAPTURE flag from soundcard ([%s] device ID [%0d] type [%s]) to use HAEC Java capture soundcard", card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type));
+			card->capabilities &= ~MS_SND_CARD_CAP_CAPTURE;
+			card_data->builtin_aec = false;
 		}
-		jmethodID getSampleRate = env->GetStaticMethodID(mediastreamerAndroidContextClass, "getDeviceFavoriteSampleRate", "()I");
-		if (getSampleRate != NULL) {
-				jint ret = env->CallStaticIntMethod(mediastreamerAndroidContextClass, getSampleRate);
-				DeviceFavoriteSampleRate = (int)ret;
-				ms_message("[OpenSLES] Using %i for sample rate value", DeviceFavoriteSampleRate);
-		}
-		env->DeleteLocalRef(mediastreamerAndroidContextClass);
 	}
 
-	OpenSLESContext *context = opensles_context_init();
-	if (d->flags & DEVICE_HAS_BUILTIN_OPENSLES_AEC) {
-		card->capabilities |= MS_SND_CARD_CAP_BUILTIN_ECHO_CANCELLER;
-		context->builtin_aec = true;
-	} else if (d->flags & DEVICE_HAS_BUILTIN_AEC && !(d->flags & DEVICE_HAS_BUILTIN_AEC_CRAPPY)) {
-		ms_warning("[OpenSLES] Removing MS_SND_CARD_CAP_CAPTURE flag from soundcard to use HAEC Java capture soundcard");
-		card->capabilities = MS_SND_CARD_CAP_PLAYBACK;
+	card->latency = deviceDescription->delay;
+	if (deviceDescription->recommended_rate){
+		card_data->samplerate = deviceDescription->recommended_rate;
 	}
-	card->latency = d->delay;
-	card->data = context;
-	if (d->recommended_rate){
-		context->samplerate = d->recommended_rate;
-	}
-	return card;
 }
 
+
+static void snd_card_device_create(int device_id, const char * name, MSSndCardDeviceType type, unsigned int capabilities, SoundDeviceDescription *deviceDescription, MSSndCardManager *m) {
+	MSSndCard *card = ms_snd_card_new(&android_native_snd_opensles_card_desc);
+	card = ms_snd_card_ref(card);
+
+	card->name = ms_strdup(name);
+	card->internal_id = device_id;
+	card->device_type = type;
+
+	// Card capabilities
+	card->capabilities = capabilities;
+	snd_card_device_create_extra_fields(m, card, deviceDescription);
+
+	bool hasCaptureCapability = (card->capabilities & MS_SND_CARD_CAP_CAPTURE) == MS_SND_CARD_CAP_CAPTURE;
+	bool hasPlaybackCapability = (card->capabilities & MS_SND_CARD_CAP_PLAYBACK) == MS_SND_CARD_CAP_PLAYBACK;
+	// Last argument is set to false because cards are added based on their type.
+	// Their capabilities are ignored as we do upcalls to the audio manager to set the desired device
+	// For example a Bluetooth device can be a speaker and/or microphone. The upcall to the audio manager will enable (i.e. start) or disable (i.e. stop) the bluetooth device regardless whether it is used as speaker or microphone.
+	MSSndCard *duplicate = ms_snd_card_get_card_duplicate(m, card, FALSE);
+	if (duplicate == NULL) {
+		if (hasCaptureCapability || hasPlaybackCapability) {
+			ms_snd_card_manager_add_card(m, card);
+			ms_message("[OpenSLES] Added card [%p]: name [%s] device ID [%0d] type [%s]", card, card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type));
+		} else {
+			ms_warning("[OpenSLES] Card [%p]: name [%s] device ID [%0d] type [%s] not added, neither capture nor playback capability", card, card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type));
+		}
+	} else {
+		ms_warning("[OpenSLES] Card [%p]: name [%s] device ID [%0d] type [%s] not added, considered a duplicate of card [%p]: name [%s] device ID [%0d] type [%s]", card, card->name, card->internal_id, ms_snd_card_device_type_to_string(card->device_type), duplicate, duplicate->name, duplicate->internal_id, ms_snd_card_device_type_to_string(duplicate->device_type));
+		if (card->device_type == MS_SND_CARD_DEVICE_TYPE_BLUETOOTH || card->device_type == MS_SND_CARD_DEVICE_TYPE_HEADSET || card->device_type == MS_SND_CARD_DEVICE_TYPE_HEADPHONES) {
+			if (hasCaptureCapability && (duplicate->capabilities & MS_SND_CARD_CAP_PLAYBACK) == MS_SND_CARD_CAP_PLAYBACK) {
+				ms_message("[OpenSLES] Duplicate card has Playback capability, adding to it the Capture capability");
+				duplicate->capabilities |= MS_SND_CARD_CAP_CAPTURE;
+			} else if (hasPlaybackCapability && (duplicate->capabilities & MS_SND_CARD_CAP_CAPTURE) == MS_SND_CARD_CAP_CAPTURE) {
+				ms_message("[OpenSLES] Duplicate card has Capture capability, adding to it the Playback capability");
+				duplicate->capabilities |= MS_SND_CARD_CAP_PLAYBACK;
+			}
+			ms_snd_card_unref(duplicate);
+		}
+	}
+	ms_snd_card_unref(card);
+}
+
+static void android_snd_card_device_create(JNIEnv *env, jobject deviceInfo, SoundDeviceDescription *deviceDescription, MSSndCardManager *m) {
+	MSSndCardDeviceType type = ms_android_get_device_type(env, deviceInfo);
+	if (
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_BLUETOOTH) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_EARPIECE) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_SPEAKER) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_MICROPHONE) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADSET) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_HEADPHONES) ||
+		(type == MSSndCardDeviceType::MS_SND_CARD_DEVICE_TYPE_GENERIC_USB)
+	) {
+
+		char *name = ms_android_get_device_product_name(env, deviceInfo);
+		int device_id = ms_android_get_device_id(env, deviceInfo);
+		unsigned int capabilities = ms_android_get_device_capabilities(env, deviceInfo);
+		snd_card_device_create(device_id, name, type, capabilities, deviceDescription, m);
+		ms_free(name);
+	} else {
+		ms_message("[OpenSLES] SKipped device with type [%s]", ms_snd_card_device_type_to_string(type));
+	}
+}
