@@ -62,7 +62,7 @@ static SoundDeviceDescription devices[]={
 	{	"alps",					"Bluboo Maya Max",		"mt6750",		DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0 },
 	{	"HTC",					"Nexus One",			"qsd8k",		0,	300 },
 	{	"HTC",					"HTC One X",			"tegra",		0,	150 },	/*has a very good acoustic isolation, which result in calibration saying no echo. */
-										/*/But with speaker mode there is a strong echo if software ec is disabled.*/
+																					/*/But with speaker mode there is a strong echo if software ec is disabled.*/
 	{	"HTC",					"HTC One SV",			"msm8960",		0,	200 },
 	{	"HTC",					"HTC Desire",			"",				0,	250 },
 	{	"HTC",					"HTC Sensation Z710e",	"",				0,	200 },
@@ -83,7 +83,7 @@ static SoundDeviceDescription devices[]={
 	{	"motorola",				"XT907",				"",				0,	500 },
 	{	"motorola",				"DROIX X2",				"",				0,	320 },
 	{	"motorola",				"MotoG3",				"msm8916",		DEVICE_HAS_BUILTIN_AEC_CRAPPY,	100 }, /*The MotoG3 audio capture hangs for several seconds when switching to speaker mode*/
-	{	"motorola",				"Moto G (4)",				"msm8952",			0, 0, 44100 }, /*the soundcard doesn't work well at 48khz, it apparently output 44100 ! */
+	{	"motorola",				"Moto G (4)",			"msm8952",		0, 0, 44100 }, /*the soundcard doesn't work well at 48khz, it apparently output 44100 ! */
 	{	"motorola",				"Nexus 6",				"msm8084",		DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0}, /* Nexus 6*/
 	{	"samsung",				"GT-S5360",				"bcm21553",		0,	250 }, /*<Galaxy Y*/
 	{	"samsung",				"GT-S5360L",			"",				0,	250 }, /*<Galaxy Y*/
@@ -188,8 +188,8 @@ static SoundDeviceDescription devices[]={
 	{	"LENOVO",				"Lenovo S60-a",			"msm8916",		0,	0 ,	44100},
 	{	"LENOVO",				"Lenovo A6000",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0},
 	{	"LENOVO",				"Lenovo A616",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0},
-	{	"LENOVO",				"Lenovo YT3-850F",		"msm8909",			DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0},
-	{	"LENOVO", 				"Lenovo TB-X704F",		"msm8953",			DEVICE_MCH265_LIMIT_DEQUEUE_OF_OUTPUT_BUFFERS, 0},
+	{	"LENOVO",				"Lenovo YT3-850F",		"msm8909",		DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0},
+	{	"LENOVO", 				"Lenovo TB-X704F",		"msm8953",		DEVICE_MCH265_LIMIT_DEQUEUE_OF_OUTPUT_BUFFERS, 0},
 
 	{	"Enspert",				"IGGY",					""		,		0,	320 ,0}, /*Wiko iggy*/
 	{	"Yota Devices Limited", "YD201",				"msm8974",		DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 48000 }, /* Yotaphone 2 */
@@ -203,7 +203,7 @@ static SoundDeviceDescription devices[]={
 
 	{	"Alcatel",				"4027D",				"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"Alcatel",				"7045Y", 				"", 			DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
-	{	"TCL",					"6070Y",			"msm8952",	DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000},
+	{	"TCL",					"6070Y",				"msm8952",		DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000},
 	
 
 	{	"HUAWEI",				"HUAWEI P7-L10",		"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
@@ -219,11 +219,13 @@ static SoundDeviceDescription devices[]={
 	{	"HUAWEI",				"HUAWEI VNS-L31",		"hi6250",		DEVICE_MCH264ENC_NO_PIX_FMT_CONV, 0 },
 	{	"HUAWEI",				"ALE-L21",				"hi6210sft",	DEVICE_MCH264ENC_NO_PIX_FMT_CONV, 0 },
 	{	"Huawei",				"AOSP on angler",		"msm8994",		DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000},
+	{	"HUAWEI",				"SLA-L02",				"msm8937",		DEVICE_HAS_BUILTIN_OPENSLES_AEC, 0, 16000 },
 
 	{	"WIKO", 				"HIGHWAY 4G",			"tegra",		DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
 	{	"Symphony",				"HIGHWAY 4G",			"",				DEVICE_HAS_UNSTANDARD_LIBMEDIA, 0 },
-	{	"rockchip",				"rk3288_box",			"rk3288",	DEVICE_USE_ANDROID_CAMCORDER, 0, 16000},
+	{	"rockchip",				"rk3288_box",			"rk3288",		DEVICE_USE_ANDROID_CAMCORDER, 0, 16000},
 	{	"rockchip",				"X9-LX",				"rk3288",		DEVICE_USE_ANDROID_CAMCORDER, 0, 16000},
+	{	"ZTE",					"Z839",					"sweet",		DEVICE_HAS_BUILTIN_AEC_CRAPPY, 0}, // Wrong microphone issue, this workarounds it
 	{	NULL, 					NULL,					NULL,			0, 	0,	0}
 };
 
@@ -246,7 +248,7 @@ JNIEXPORT void JNICALL Java_org_linphone_mediastream_MediastreamerAndroidContext
 
 #else
 
-static SoundDeviceDescription devices[]={
+static const SoundDeviceDescription devices[]={
 	{ NULL, NULL, NULL, 0, 0, 0, NULL }
 };
 
@@ -291,7 +293,7 @@ void ms_sound_device_description_add(const char *manufacturer, const char *model
 MSDevicesInfo *ms_devices_info_new(void) {
 	MSDevicesInfo *devices_info = ms_new0(MSDevicesInfo, 1);
 
-	SoundDeviceDescription *d = &devices[0];
+	const SoundDeviceDescription *d = &devices[0];
 	while (d->manufacturer != NULL) {
 		ms_devices_info_add(devices_info, d->manufacturer, d->model, d->platform, d->flags, d->delay, d->recommended_rate);
 		d++;
@@ -346,6 +348,7 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 	bool_t declares_builtin_aec = FALSE;
 
 #ifdef __ANDROID__
+	bool_t force_disable_builtin_aec = FALSE;
 
 	if (__system_property_get("ro.product.manufacturer", manufacturer) <= 0) {
 		ms_warning("Could not get product manufacturer.");
@@ -368,7 +371,10 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 				if (ret) {
 					ms_message("This device (%s/%s/%s) declares it has a built-in echo canceller.", manufacturer, model, platform);
 					declares_builtin_aec = TRUE;
-				} else ms_message("This device (%s/%s/%s) says it has no built-in echo canceller.", manufacturer, model, platform);
+				} else {
+					ms_message("This device (%s/%s/%s) says it has no built-in echo canceller.", manufacturer, model, platform);
+					force_disable_builtin_aec = TRUE;
+				}
 			} else {
 				ms_error("isAvailable() not found in class AcousticEchoCanceler !");
 				(*env)->ExceptionClear(env); //very important.
@@ -385,10 +391,18 @@ SoundDeviceDescription* ms_devices_info_get_sound_device_description(MSDevicesIn
 	if (!d) {
 		ms_message("No information available for [%s/%s/%s],", manufacturer, model, platform);
 #ifdef __ANDROID__
-		if (ms2_android_get_sdk_version() >= 26){
-			d = &genericSoundDeviceDescriptorAboveAndroid8;
-			ms_message("Using android >= 8 sound device descriptor.");
-		}else d = &genericSoundDeviceDescriptor;
+		if (ms2_android_get_sdk_version() >= 26) {
+			if (force_disable_builtin_aec) {
+				d = &genericSoundDeviceDescriptor;
+				ms_message("Android >= 8 but AcousticEchoCanceler isn't available, using default sound device descriptor");
+			} else {
+				d = &genericSoundDeviceDescriptorAboveAndroid8;
+				ms_message("Android >= 8, using sound device descriptor.with DEVICE_HAS_BUILTIN_AEC | DEVICE_HAS_BUILTIN_OPENSLES_AEC flags");
+			}
+		} else {
+			d = &genericSoundDeviceDescriptor;
+			ms_message("Android < 8, using default sound device descriptor");
+		}
 #else
 		d = &genericSoundDeviceDescriptor;
 #endif
