@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #include "mediastreamer2/msfilter.h"
 #include "mediastreamer2/msticker.h"
@@ -36,6 +36,7 @@ typedef struct SpeexEncState{
 	int maxbitrate; /*ip bitrate*/
 	int ip_bitrate; /*effective ip bitrate */
 	int ptime;
+	int maxptime;
 	int vbr;
 	int cng;
 	int mode;
@@ -55,6 +56,7 @@ static void enc_init(MSFilter *f){
 	s->maxbitrate=-1;
 	s->ip_bitrate=-1;
 	s->ptime=20;
+	s->maxptime=MS_DEFAULT_MAX_PTIME;
 	s->mode=-1;
 	s->vbr=0;
 	s->cng=0;
@@ -319,6 +321,7 @@ static int enc_set_ptime(MSFilter *f, void *arg){
 	/*if the ptime is not a mulptiple of 20, go to the next multiple*/
 	if (s->ptime%20)
 		s->ptime = s->ptime - s->ptime%20 + 20;
+	s->ptime = MIN(s->ptime,s->maxptime);
 	ms_message("MSSpeexEnc: got ptime=%i",s->ptime);
 	return 0;
 }
@@ -376,6 +379,12 @@ static int enc_add_fmtp(MSFilter *f, void *arg){
 		s->mode = -1; /* default mode */
 	}
 	memset(buf, '\0', sizeof(buf));
+	
+	if (fmtp_get_value(fmtp,"maxptime",buf,sizeof(buf))){
+		s->maxptime=atoi(buf);
+		ms_message("MSSpeexEnc: got maxptime=%i",s->maxptime);
+	}
+	
 	if (fmtp_get_value(fmtp,"ptime",buf,sizeof(buf))){
 		int val=atoi(buf);
 		enc_set_ptime(f,&val);
