@@ -229,7 +229,7 @@ static void pulse_card_source_create(pa_device_t *pa_device, MSSndCardManager *m
  * output: sink and bidirectionnal cards into sink list, source only card into source list
  * merging is based on pulse audio card description */
 int pulse_card_compare(pa_device_t *sink, pa_device_t *source) {
-	return strncmp(sink->description, source->description, 512);
+	return strncmp(sink->description, source->description, PA_STRING_SIZE);
 }
 
 static void pulse_card_merge_lists(pa_device_t *pa_device, bctbx_list_t **pa_source_list) {
@@ -457,10 +457,8 @@ static bool_t stream_connect(Stream *s) {
 	pa_threaded_mainloop_unlock(the_pa_loop);
 	if(err < 0 || !stream_wait_for_state(s, PA_STREAM_READY, PA_STREAM_FAILED)) {
 		ms_error("Fails to connect pulseaudio stream. err=%d", err);
-		pa_threaded_mainloop_lock(the_pa_loop);
 		pa_stream_unref(s->stream);
 		s->stream = NULL;
-		pa_threaded_mainloop_unlock(the_pa_loop);
 		return FALSE;
 	}
 	ms_message("pulseaudio %s stream connected (%dHz, %dch)",
@@ -480,10 +478,9 @@ static void stream_disconnect(Stream *s) {
 		if(err!=0 || !stream_wait_for_state(s, PA_STREAM_TERMINATED, PA_STREAM_FAILED)) {
 			ms_error("pa_stream_disconnect() failed. err=%d", err);
 		}
-		pa_threaded_mainloop_lock(the_pa_loop);
 		pa_stream_unref(s->stream);
+		
 		s->stream = NULL;
-		pa_threaded_mainloop_unlock(the_pa_loop);
 		s->state = PA_STREAM_UNCONNECTED;
 		s->init_volume = -1.0;
 	}
@@ -510,9 +507,7 @@ static bool_t stream_set_volume(Stream *s, double volume) {
 			return TRUE;
 		}
 	}
-	pa_threaded_mainloop_lock(the_pa_loop);
 	idx = pa_stream_get_index(s->stream);
-	pa_threaded_mainloop_unlock(the_pa_loop);
 	pa_cvolume_init(&cvolume);
 	pa_cvolume_set(&cvolume, s->sampleSpec.channels, scale_to_volume(volume));
 	pa_threaded_mainloop_lock(the_pa_loop);
@@ -551,9 +546,9 @@ static bool_t stream_get_volume(Stream *s, double *volume) {
 		ms_error("stream_get_volume(): no stream");
 		return FALSE;
 	}
-	pa_threaded_mainloop_lock(the_pa_loop);
 	idx = pa_stream_get_index(s->stream);
 	*volume = -1.0;
+	pa_threaded_mainloop_lock(the_pa_loop);
 	if(s->type == STREAM_TYPE_PLAYBACK) {
 		op = pa_context_get_sink_input_info(the_pa_context, idx, stream_get_sink_volume_cb, volume);
 	} else {
@@ -620,12 +615,11 @@ static void pulse_read_process(MSFilter *f){
 			if (buffer == NULL) ms_message("%s: hole detected while reading samples. %zu bytes of data have been lost", f->desc->name, nbytes);
 			// ticker synchronization must be done here to ensure that lost samples are taken into account while computing total read samples count.
 			s->read_samples += ((nbytes / 2) / s->sampleSpec.channels);
-			ms_ticker_synchronizer_update(s->ticker_synchronizer, s->read_samples, (unsigned int)s->sampleSpec.rate);
 			pa_stream_drop(s->stream);
 		}
 	}
-
 	pa_threaded_mainloop_unlock(the_pa_loop);
+	ms_ticker_synchronizer_update(s->ticker_synchronizer, s->read_samples, (unsigned int)s->sampleSpec.rate);
 }
 
 static void pulse_read_postprocess(MSFilter *f) {
