@@ -158,11 +158,19 @@ int ms_ticker_attach_multiple(MSTicker *ticker,MSFilter *f,...)
 				break;
 			}
 			/*run preprocess on each filter: */
-			for(it=filters;it!=NULL;it=it->next)
+			for(it=filters;it!=NULL;it=it->next) {
 				ms_filter_preprocess((MSFilter*)it->data,ticker);
+			}
 			bctbx_list_free(filters);
 			total_sources=bctbx_list_concat(total_sources,sources);
-		}else ms_message("Filter %s is already being scheduled; nothing to do.",f->desc->name);
+		}else{
+			if (f->ticker == ticker){
+				ms_message("Filter %s is already being scheduled; nothing to do.",f->desc->name);
+			}else{
+				ms_fatal("MSTicker %p; cannot attach filter %s:%p : it is already being run by ticker %p.",
+					 ticker, f->desc->name, f, f->ticker);
+			}
+		}
 	}while ((f=va_arg(l,MSFilter*))!=NULL);
 	va_end(l);
 	if (total_sources){
@@ -178,7 +186,7 @@ static void call_postprocess(MSFilter *f){
 	ms_filter_postprocess(f);
 }
 
-int ms_ticker_detach(MSTicker *ticker,MSFilter *f){
+int ms_ticker_detach(MSTicker *ticker, MSFilter *f){
 	bctbx_list_t *sources=NULL;
 	bctbx_list_t *filters=NULL;
 	bctbx_list_t *it;
@@ -187,6 +195,12 @@ int ms_ticker_detach(MSTicker *ticker,MSFilter *f){
 		ms_message("Filter %s is not scheduled; nothing to do.",f->desc->name);
 		return 0;
 	}
+	
+	if (f->ticker != ticker){
+		ms_fatal("ms_ticker_detach(): filter %s:%p is currently scheduled by MSTicker %p, but requested to detach from MSTicker %p. This is a programming mistake.",
+			 f->desc->name, f, f->ticker, ticker);
+		return -1;
+	}
 
 	ms_mutex_lock(&ticker->lock);
 
@@ -312,7 +326,7 @@ static int set_high_prio(MSTicker *obj){
 
 	if (prio>MS_TICKER_PRIO_NORMAL){
 #ifdef _WIN32
-#ifdef MS2_WINDOWS_DESKTOP
+#if defined(MS2_WINDOWS_DESKTOP) && !defined(MS2_WINDOWS_UWP)
 		MMRESULT mm;
 		TIMECAPS ptc;
 		mm=timeGetDevCaps(&ptc,sizeof(ptc));
@@ -333,6 +347,8 @@ static int set_high_prio(MSTicker *obj){
 		if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST)){
 			ms_warning("SetThreadPriority() failed (%d)\n", (int)GetLastError());
 		}
+#else
+		ms_warning("SetThreadPriority() is not implemented. %s priority left to normal.",obj->name);
 #endif
 #else
 		struct sched_param param;
@@ -380,7 +396,7 @@ static int set_high_prio(MSTicker *obj){
 
 static void unset_high_prio(int precision){
 #ifdef _WIN32
-#ifdef MS2_WINDOWS_DESKTOP
+#if defined(MS2_WINDOWS_DESKTOP) && !defined(MS2_WINDOWS_UWP)
 	if(!SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL)){
 		ms_warning("SetThreadPriority() failed (%d)\n", (int)GetLastError());
 	}
@@ -403,7 +419,7 @@ static int wait_next_tick(void *data, uint64_t virt_ticker_time){
 		diff=s->time-realtime;
 		if (diff>0){
 			/* sleep until next tick */
-			ortp_sleep_ms((int)diff);
+            ortp_sleep_ms((int)diff < 10 ? (int)diff : 10);
 		}else{
 			late=(int)-diff;
 			break; /*exit the while loop */
@@ -416,9 +432,11 @@ static int wait_next_tick(void *data, uint64_t virt_ticker_time){
 void * ms_ticker_run(void *arg)
 {
 	MSTicker *s=(MSTicker*)arg;
-	int lastlate=0;
+	int last_late=0;
 	int precision=2;
 	int late;
+	int max_late=0;
+	uint64_t late_time_checkpoint=0;
 
 	ms_mutex_lock(&s->lock);
 
@@ -430,7 +448,7 @@ void * ms_ticker_run(void *arg)
 	ms_mutex_unlock(&s->cur_time_lock);
 
 	while(s->run){
-		uint64_t late_tick_time=0;
+		uint64_t late_tick_time=0, current_time;
 
 		s->ticks++;
 		/*Step 1: run the graphs*/
@@ -453,11 +471,19 @@ void * ms_ticker_run(void *arg)
 		/*Step 2: wait for next tick*/
 		s->time+=s->interval;
 		late=s->wait_next_tick(s->wait_next_tick_data,s->time);
-		if (late>s->interval*5 && late>lastlate){
-			ms_warning("%s: We are late of %d miliseconds.",s->name,late);
-			late_tick_time=ms_get_cur_time_ms();
+		current_time = ms_get_cur_time_ms();
+		if(current_time > late_time_checkpoint + 1000) {// We print max late each 1s
+			if( max_late > 0){// We have a counted late
+				ms_warning("%s: We are late of %d miliseconds.",s->name,max_late);
+				max_late = 0;// Reset max
+			}
+			late_time_checkpoint = current_time;// Reset time
 		}
-		lastlate=late;
+		if (late>s->interval*5 && late>last_late){
+			max_late = (late > max_late? late : max_late);// Get the max
+			late_tick_time=current_time;
+		}
+		last_late=late;
 		ms_mutex_lock(&s->lock);
 		if (late_tick_time){
 			s->late_event.lateMs=late;
@@ -606,7 +632,7 @@ double ms_ticker_synchronizer_set_external_time(MSTickerSynchronizer* ts, const
 	uint64_t ms = get_ms(time);
 	if (ts->offset == 0) {
 		/* In case of resync, the av_skew must be preserved */
-		ts->offset = wc - ms - (uint64_t)ts->av_skew;
+		ts->offset = wc - ms - (int64_t)ts->av_skew;
 	}
 	sound_time = ts->offset + ms;
 	diff = wc - sound_time;
