@@ -56,7 +56,16 @@ struct SenderData {
 	bool_t mute;
 	bool_t use_task;
 	bool_t stun_enabled;
+	bool_t stun_forced_enabled;
 	bool_t enable_ts_adjustment;
+	int audio_level_send_interval;
+	uint64_t last_audio_level_sent_time;
+	int mixer_to_client_extension_id;
+	MSRtpSendRequestMixerToClientDataCb mtc_request_data_cb;
+	void *mtc_request_data_user_data;
+	int client_to_mixer_extension_id;
+	MSRtpSendRequestClientToMixerDataCb ctm_request_data_cb;
+	void *ctm_request_data_user_data;
 };
 
 typedef struct SenderData SenderData;
@@ -70,7 +79,7 @@ static void send_stun_packet(SenderData *d, bool_t enable_rtp, bool_t enable_rtc
 	char *buf = NULL;
 	size_t len;
 
-	if (!d->stun_enabled) return;
+	if (!d->stun_enabled && !d->stun_forced_enabled) return;
 	if (ms_is_multicast_addr((const struct sockaddr *)&s->rtcp.gs.loc_addr)) {
 		ms_debug("Stun packet not sent for session [%p] because of multicast",s);
 		return;
@@ -83,14 +92,15 @@ static void send_stun_packet(SenderData *d, bool_t enable_rtp, bool_t enable_rtc
 			mp = allocb(len, BPRI_MED);
 			memcpy(mp->b_wptr, buf, len);
 			mp->b_wptr += len;
-			ms_message("Stun packet sent for session [%p]",s);
+
+			ms_message("Stun packet of length %0zd sent on rtp for session [%p] %s",len, s, d->stun_forced_enabled ?  "(forced)" : "");
 			rtp_session_sendm_with_ts(s, mp, 0);
 		}
 		if (enable_rtcp) {
 			mp = allocb(len, BPRI_MED);
 			memcpy(mp->b_wptr, buf, len);
 			mp->b_wptr += len;
-			ms_message("Stun packet sent on rtcp for session [%p]",s);
+			ms_message("Stun packet of length %0zd sent on rtcp for session [%p] %s",len, s, d->stun_forced_enabled ?  "(forced)" : "");
 			rtp_session_rtcp_sendm_raw(s,mp);
 		}
 	}
@@ -126,6 +136,14 @@ static void sender_init(MSFilter * f)
 	if (d->use_task) ms_message("MSRtpSend will use tasks to send out packet at the beginning of ticks.");
 	d->stun_enabled = TRUE;
 	d->enable_ts_adjustment = TRUE;
+	d->audio_level_send_interval = 250;
+	d->last_audio_level_sent_time = 0;
+	d->mixer_to_client_extension_id = 0;
+	d->mtc_request_data_cb = NULL;
+	d->mtc_request_data_user_data = NULL;
+	d->client_to_mixer_extension_id = 0;
+	d->ctm_request_data_cb = NULL;
+	d->ctm_request_data_user_data = NULL;
 	f->data = d;
 }
 
@@ -206,6 +224,43 @@ static int sender_set_relay_session_id(MSFilter *f, void*arg){
 	return 0;
 }
 
+static int sender_set_audio_level_send_interval(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	int *interval = (int *)arg;
+	d->audio_level_send_interval = *interval;
+	return 0;
+}
+
+static int sender_set_mixer_to_client_extension_id(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	int *id = (int *)arg;
+	d->mixer_to_client_extension_id = *id;
+	return 0;
+}
+
+static int sender_set_mixer_to_client_data_request_cb(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	MSFilterRequestMixerToClientDataCb *cb = (MSFilterRequestMixerToClientDataCb *)arg;
+	d->mtc_request_data_cb = cb->cb;
+	d->mtc_request_data_user_data = cb->user_data;
+	return 0;
+}
+
+static int sender_set_client_to_mixer_extension_id(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	int *id = (int *)arg;
+	d->client_to_mixer_extension_id = *id;
+	return 0;
+}
+
+static int sender_set_client_to_mixer_data_request_cb(MSFilter *f, void *arg) {
+	SenderData *d = (SenderData *) f->data;
+	MSFilterRequestClientToMixerDataCb *cb = (MSFilterRequestClientToMixerDataCb *)arg;
+	d->ctm_request_data_cb = cb->cb;
+	d->ctm_request_data_user_data = cb->user_data;
+	return 0;
+}
+
 static int sender_get_sr(MSFilter *f, void *arg){
 	SenderData *d = (SenderData *) f->data;
 	PayloadType *pt;
@@ -353,7 +408,10 @@ static int send_dtmf(MSFilter * f, uint32_t timestamp_start)
 	
 	m1=rtp_session_create_telephone_event_packet(d->session,timestamp_start==d->dtmf_ts_cur);
 	
-	if (m1==NULL) return -1;
+	if (m1==NULL) {
+		ms_warning("There is no usable telephone-event payload type configured in the RTP session. The DTMF cannot be sent.");
+		return -1;
+	}
 
 	d->dtmf_ts_cur+=d->dtmf_ts_step;
 	if (RTP_TIMESTAMP_IS_NEWER_THAN(d->dtmf_ts_cur, d->skip_until)) {
@@ -384,8 +442,11 @@ static void check_stun_sending(MSFilter *f) {
 	uint64_t last_sent_timeout = 20000;
 	if (rtp_session_get_stats(s)->packet_sent == 0)
 		last_sent_timeout = 2000;
-	if ((d->last_rtp_stun_sent_time == -1) || (((f->ticker->time - d->last_sent_time) > last_sent_timeout)
+	if ((d->last_rtp_stun_sent_time == -1) || (  (d->stun_forced_enabled ||  ((f->ticker->time - d->last_sent_time) > last_sent_timeout))
 											&& (f->ticker->time - d->last_rtp_stun_sent_time) >= 500)) {
+		/* send stun packet every 500 ms: 
+		 * - in absence of any RTP packet for more than last_sent_timeout
+		 * - or always in "forced" mode */
 		d->last_rtp_stun_sent_time = f->ticker->time;
 		send_stun_packet(d,TRUE,FALSE);
 	}
@@ -440,7 +501,7 @@ static void _sender_process(MSFilter * f)
 	RtpSession *s = d->session;
 	mblk_t *im;
 	uint32_t timestamp;
-
+	rtp_audio_level_t audio_levels[RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL] = {{0}};
 
 	if (d->relay_session_id_size>0 && 
 		( (f->ticker->time-d->last_rsi_time)>5000 || d->last_rsi_time==0) ) {
@@ -452,7 +513,7 @@ static void _sender_process(MSFilter * f)
 	ms_filter_lock(f);
 	im = ms_queue_get(f->inputs[0]);
 	do {
-		mblk_t *header;
+		mblk_t *header = NULL;
 
 		timestamp = get_cur_timestamp(f, im);
 		
@@ -466,13 +527,33 @@ static void _sender_process(MSFilter * f)
 			uint32_t origin_ts=d->skip_until-d->dtmf_duration;
 			if (RTP_TIMESTAMP_IS_NEWER_THAN(timestamp,d->dtmf_ts_cur)){
 				ms_debug("Sending RFC2833 packet, start_timestamp=%u, dtmf_ts_cur=%u",origin_ts,d->dtmf_ts_cur);
-				send_dtmf(f, origin_ts);
+				if (send_dtmf(f, origin_ts) == -1){
+					/* Abort the sending of dtmf if an error occured */
+					d->dtmf = 0;
+					d->skip = FALSE;
+				}
 			}
 		}
 		if (im){
 			compute_processing_delay_stats(f, im);
 			if (d->skip == FALSE && d->mute==FALSE){
-				header = rtp_session_create_packet(s, 12, NULL, 0);
+				if ((d->mixer_to_client_extension_id > 0 || d->client_to_mixer_extension_id > 0)
+					&& f->ticker->time - d->last_audio_level_sent_time > (uint64_t)d->audio_level_send_interval) {
+
+					if (d->mixer_to_client_extension_id > 0 && d->mtc_request_data_cb) {
+						int size = d->mtc_request_data_cb(f, audio_levels, d->mtc_request_data_user_data);
+						header = rtp_session_create_packet_with_mixer_to_client_audio_level(s, 12, d->mixer_to_client_extension_id, size, audio_levels);
+					}
+					if (d->client_to_mixer_extension_id > 0 && d->ctm_request_data_cb) {
+						int volume = d->ctm_request_data_cb(f, d->ctm_request_data_user_data);
+						if (!header) header = rtp_session_create_packet(s, 12, NULL, 0);
+						rtp_add_client_to_mixer_audio_level(header, d->client_to_mixer_extension_id, FALSE, volume);
+					}
+
+					d->last_audio_level_sent_time = f->ticker->time;
+				}
+
+				if (!header) header = rtp_session_create_packet(s, 12, NULL, 0);
 				rtp_set_markbit(header, mblk_get_marker_info(im));
 				header->b_cont = im;
 				mblk_meta_copy(im, header);
@@ -531,6 +612,12 @@ static int sender_enable_stun(MSFilter *f, void *data) {
 	return 0;
 }
 
+static int sender_enable_stun_forced(MSFilter *f, void *data) {
+	SenderData *d = (SenderData *)f->data;
+	d->stun_forced_enabled = *((bool_t *)data);
+	return 0;
+}
+
 static int get_sender_output_fmt(MSFilter *f, void *arg) {
 	SenderData *d = (SenderData *) f->data;
 	MSPinFormat *pinFmt = (MSPinFormat *)arg;
@@ -551,6 +638,11 @@ static MSFilterMethod sender_methods[] = {
 	{MS_RTP_SEND_SET_SESSION, sender_set_session},
 	{MS_RTP_SEND_SEND_DTMF, sender_send_dtmf},
 	{MS_RTP_SEND_SET_RELAY_SESSION_ID, sender_set_relay_session_id},
+	{MS_RTP_SEND_SET_AUDIO_LEVEL_SEND_INTERVAL, sender_set_audio_level_send_interval},
+	{MS_RTP_SEND_SET_MIXER_TO_CLIENT_EXTENSION_ID, sender_set_mixer_to_client_extension_id},
+	{MS_RTP_SEND_SET_MIXER_TO_CLIENT_DATA_REQUEST_CB, sender_set_mixer_to_client_data_request_cb},
+	{MS_RTP_SEND_SET_CLIENT_TO_MIXER_EXTENSION_ID, sender_set_client_to_mixer_extension_id},
+	{MS_RTP_SEND_SET_CLIENT_TO_MIXER_DATA_REQUEST_CB, sender_set_client_to_mixer_data_request_cb},
 	{MS_FILTER_GET_SAMPLE_RATE, sender_get_sr },
 	{MS_FILTER_GET_NCHANNELS, sender_get_ch },
 	{MS_RTP_SEND_SET_DTMF_DURATION, sender_set_dtmf_duration },
@@ -558,6 +650,7 @@ static MSFilterMethod sender_methods[] = {
 	{ MS_RTP_SEND_ENABLE_STUN, sender_enable_stun },
 	{ MS_FILTER_GET_OUTPUT_FMT, get_sender_output_fmt },
 	{ MS_RTP_SEND_ENABLE_TS_ADJUSTMENT, enable_ts_adjustment },
+	{ MS_RTP_SEND_ENABLE_STUN_FORCED, sender_enable_stun_forced },
 	{0, NULL}
 };
 
@@ -604,6 +697,8 @@ struct ReceiverData {
 	int rate;
 	bool_t starting;
 	bool_t reset_jb;
+	int mixer_to_client_extension_id;
+	int client_to_mixer_extension_id;
 };
 
 typedef struct ReceiverData ReceiverData;
@@ -613,6 +708,7 @@ static void receiver_init(MSFilter * f)
 	ReceiverData *d = (ReceiverData *)ms_new0(ReceiverData, 1);
 	d->session = NULL;
 	d->rate = 8000;
+	d->mixer_to_client_extension_id = 0;
 	f->data = d;
 }
 
@@ -644,6 +740,20 @@ static int receiver_set_session(MSFilter * f, void *arg)
 	return 0;
 }
 
+static int receiver_set_mixer_to_client_extension_id(MSFilter * f, void *arg) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	int *id = (int *) arg;
+	d->mixer_to_client_extension_id = *id;
+	return 0;
+}
+
+static int receiver_set_client_to_mixer_extension_id(MSFilter * f, void *arg) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	int *id = (int *) arg;
+	d->client_to_mixer_extension_id = *id;
+	return 0;
+}
+
 static int receiver_get_sr(MSFilter *f, void *arg){
 	ReceiverData *d = (ReceiverData *) f->data;
 	PayloadType *pt;
@@ -721,6 +831,29 @@ static bool_t receiver_check_payload_type(MSFilter *f, ReceiverData *d, mblk_t *
 	return TRUE;
 }
 
+static void receiver_check_for_extensions(MSFilter *f, mblk_t *m) {
+	ReceiverData *d = (ReceiverData *) f->data;
+	rtp_audio_level_t mtc_levels[RTP_MAX_MIXER_TO_CLIENT_AUDIO_LEVEL] = {{0}};
+	rtp_audio_level_t ctm_level;
+	bool_t voice_activity;
+	int ret;
+
+	// Check the packet if it contains audio level extensions
+	if (d->mixer_to_client_extension_id > 0) {
+		if ((ret = rtp_get_mixer_to_client_audio_level(m, d->mixer_to_client_extension_id, mtc_levels)) != -1) {
+			ms_filter_notify(f, MS_RTP_RECV_MIXER_TO_CLIENT_AUDIO_LEVEL_RECEIVED, mtc_levels);
+		}
+	}
+
+	if (d->client_to_mixer_extension_id > 0) {
+		if ((ret = rtp_get_client_to_mixer_audio_level(m, RTP_EXTENSION_CLIENT_TO_MIXER_AUDIO_LEVEL, &voice_activity)) != -1) {
+			ctm_level.csrc = rtp_get_ssrc(m);
+			ctm_level.dbov = ret;
+			ms_filter_notify(f, MS_RTP_RECV_CLIENT_TO_MIXER_AUDIO_LEVEL_RECEIVED, &ctm_level);
+		}
+	}
+}
+
 static void receiver_process(MSFilter * f)
 {
 	ReceiverData *d = (ReceiverData *) f->data;
@@ -751,6 +884,7 @@ static void receiver_process(MSFilter * f)
 			mblk_set_timestamp_info(m, rtp_get_timestamp(m));
 			mblk_set_marker_info(m, rtp_get_markbit(m));
 			mblk_set_cseq(m, rtp_get_seqnumber(m));
+			receiver_check_for_extensions(f, m);
 			rtp_get_payload(m,&m->b_rptr);
 			ms_queue_put(f->outputs[0], m);
 		}else{
@@ -773,6 +907,8 @@ static int get_receiver_output_fmt(MSFilter *f, void *arg) {
 static MSFilterMethod receiver_methods[] = {
 	{	MS_RTP_RECV_SET_SESSION	, receiver_set_session	},
 	{	MS_RTP_RECV_RESET_JITTER_BUFFER, receiver_reset_jitter_buffer },
+	{	MS_RTP_RECV_SET_MIXER_TO_CLIENT_EXTENSION_ID, receiver_set_mixer_to_client_extension_id },
+	{	MS_RTP_RECV_SET_CLIENT_TO_MIXER_EXTENSION_ID, receiver_set_client_to_mixer_extension_id },
 	{	MS_FILTER_GET_SAMPLE_RATE	, receiver_get_sr		},
 	{	MS_FILTER_GET_NCHANNELS	,	receiver_get_ch	},
 	{ 	MS_FILTER_GET_OUTPUT_FMT, get_receiver_output_fmt },
