@@ -120,12 +120,12 @@ int avcodec_encode_video2 (AVCodecContext *avctx, AVPacket *avpkt, const AVFrame
 #endif
 
 #ifndef HAVE_FUN_avcodec_get_context_defaults3 /**/
-int avcodec_get_context_defaults3 (AVCodecContext *s, const AVCodec *codec);
+int avcodec_get_context_defaults3 (AVCodecContext *s, AVCodec *codec);
 AVCodecContext *avcodec_alloc_context3(AVCodec *codec);
 #endif
 
 #ifndef HAVE_FUN_avcodec_open2 /**/
-int avcodec_open2 (AVCodecContext *avctx, const AVCodec *codec, /*AVDictionary*/ void **options);
+int avcodec_open2 (AVCodecContext *avctx, AVCodec *codec, AVDictionary **options);
 #endif
 
 #ifndef HAVE_FUN_av_frame_alloc
