@@ -27,6 +27,7 @@
 #include "mediastreamer2/mswebcam.h"
 #include "nowebcam.h"
 #include "ioshardware.h"
+#include "apple_utils.h"
 
 #import <AVFoundation/AVFoundation.h>
 #import <UIKit/UIKit.h>
@@ -524,39 +525,43 @@ static AVCaptureVideoOrientation Angle2AVCaptureVideoOrientation(int deviceOrien
 
 - (void)setFps:(float) value {
 	@synchronized(self) {
-		AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)self.layer session];
-		[session beginConfiguration];
+		@try {
+			AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)self.layer session];
+			[session beginConfiguration];
 
-		if( [[[UIDevice currentDevice] systemVersion] floatValue] >= 7 ){
-			for( AVCaptureDeviceInput* devinput in [session inputs] ) {
+			if( [[[UIDevice currentDevice] systemVersion] floatValue] >= 7 ){
+				for( AVCaptureDeviceInput* devinput in [session inputs] ) {
 
-				NSError* err = nil;
-				if( [devinput.device lockForConfiguration:&err] == YES ){
+					NSError* err = nil;
+					if( [devinput.device lockForConfiguration:&err] == YES ){
 
-					[devinput.device setActiveVideoMinFrameDuration:CMTimeMake(1, value)];
-					[devinput.device setActiveVideoMaxFrameDuration:CMTimeMake(1, value)];
+						[devinput.device setActiveVideoMinFrameDuration:CMTimeMake(1, value)];
+						[devinput.device setActiveVideoMaxFrameDuration:CMTimeMake(1, value)];
 
-					[devinput.device unlockForConfiguration];
-				} else {
-					ms_error("Couldn't obtain lock to set capture FPS: %s", err?[err.description UTF8String] : "");
+						[devinput.device unlockForConfiguration];
+					} else {
+						ms_error("Couldn't obtain lock to set capture FPS: %s", err?[err.description UTF8String] : "");
+					}
+
+					break;
 				}
+			} else {
+				// Pre-iOS7 method
+				NSArray *connections = output.connections;
+				if ([connections count] > 0) {
+					[[connections objectAtIndex:0] setVideoMinFrameDuration:CMTimeMake(1, value)];
+					[[connections objectAtIndex:0] setVideoMaxFrameDuration:CMTimeMake(1, value)];
+				}
+			}
 
-				break;
-			}
-		} else {
-			// Pre-iOS7 method
-			NSArray *connections = output.connections;
-			if ([connections count] > 0) {
-				[[connections objectAtIndex:0] setVideoMinFrameDuration:CMTimeMake(1, value)];
-				[[connections objectAtIndex:0] setVideoMaxFrameDuration:CMTimeMake(1, value)];
-			}
+			fps=value;
+			snprintf(fps_context, sizeof(fps_context), "Captured mean fps=%%f, expected=%f", fps);
+			ms_video_init_framerate_controller(&framerate_controller, fps);
+			ms_video_init_average_fps(&averageFps, fps_context);
+			[session commitConfiguration];
+		} @catch (NSException *exception) {
+			ms_error("Exception while setting FPS with value %f: %s", value, [exception.reason UTF8String]);
 		}
-
-		fps=value;
-		snprintf(fps_context, sizeof(fps_context), "Captured mean fps=%%f, expected=%f", fps);
-		ms_video_init_framerate_controller(&framerate_controller, fps);
-		ms_video_init_average_fps(&averageFps, fps_context);
-		[session commitConfiguration];
 	}
 }
 
@@ -592,7 +597,8 @@ static AVCaptureVideoOrientation Angle2AVCaptureVideoOrientation(int deviceOrien
 
 static void ioscapture_init(MSFilter *f) {
 	NSAutoreleasePool* myPool = [[NSAutoreleasePool alloc] init];
-	f->data = [[IOSCapture alloc] initWithFrame:CGRectMake(0, 0, 0, 0) andFilter:f];
+	DISPATCH_SYNC_MAIN(^{
+		f->data = [[IOSCapture alloc] initWithFrame:CGRectMake(0, 0, 0, 0) andFilter:f];});
 	[myPool drain];
 }
 
@@ -601,10 +607,11 @@ static void ioscapture_uninit(MSFilter *f) {
 
 	if(thiz != nil) {
 		NSAutoreleasePool* myPool = [[NSAutoreleasePool alloc] init];
-		[thiz stop];
+		DISPATCH_SYNC_MAIN(^{
+			[thiz stop];
 
-		[thiz setParentView:nil];
-		[thiz release];
+			[thiz setParentView:nil];
+			[thiz release];});
 		[myPool drain];
 	}
 }
@@ -629,22 +636,23 @@ static void ioscapture_process(MSFilter * obj) {
 static void ioscapture_preprocess(MSFilter *f) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		NSAutoreleasePool* myPool = [[NSAutoreleasePool alloc] init];
-		[thiz start];
-		[myPool drain];
+		DISPATCH_SYNC_MAIN(^{
+			NSAutoreleasePool* myPool = [[NSAutoreleasePool alloc] init];
+			[thiz start];
+			[myPool drain];
 
-		AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)thiz.layer session];
-		if (!session.running) {
-			thiz->startSessionTimer = [NSTimer scheduledTimerWithTimeInterval:100 / 1000.0
-																	   target:thiz
-																	 selector:@selector(startSessionTimerFired:)
-																	 userInfo:nil
-																	  repeats:YES];
-		} else {
-			ms_mutex_lock(&thiz->mutex);
-			thiz->filterIsRunning = 1;
-			ms_mutex_unlock(&thiz->mutex);
-		}
+			AVCaptureSession *session = [(AVCaptureVideoPreviewLayer *)thiz.layer session];
+			if (!session.running) {
+				thiz->startSessionTimer = [NSTimer scheduledTimerWithTimeInterval:100 / 1000.0
+																		   target:thiz
+																		 selector:@selector(startSessionTimerFired:)
+																		 userInfo:nil
+																		  repeats:YES];
+			} else {
+				ms_mutex_lock(&thiz->mutex);
+				thiz->filterIsRunning = 1;
+				ms_mutex_unlock(&thiz->mutex);
+			}});
 	}
 }
 
@@ -665,7 +673,8 @@ static void ioscapture_postprocess(MSFilter *f) {
 static int ioscapture_get_fps(MSFilter *f, void *arg) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		*((float*)arg) = ms_average_fps_get(&thiz->averageFps);
+		DISPATCH_SYNC_MAIN(^{
+			*((float*)arg) = ms_average_fps_get(&thiz->averageFps);});
 	}
 	return 0;
 }
@@ -673,7 +682,8 @@ static int ioscapture_get_fps(MSFilter *f, void *arg) {
 static int ioscapture_set_fps(MSFilter *f, void *arg) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		[thiz setFps:*(float*)arg];
+		DISPATCH_SYNC_MAIN(^{
+			[thiz setFps:*(float*)arg];});
 	}
 	return 0;
 }
@@ -686,7 +696,8 @@ static int ioscapture_get_pix_fmt(MSFilter *f,void *arg) {
 static int ioscapture_set_vsize(MSFilter *f, void *arg) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		[thiz setSize:*((MSVideoSize*)arg)];
+		DISPATCH_SYNC_MAIN(^{
+			[thiz setSize:*((MSVideoSize*)arg)];});
 	}
 	return 0;
 }
@@ -694,7 +705,8 @@ static int ioscapture_set_vsize(MSFilter *f, void *arg) {
 static int ioscapture_get_vsize(MSFilter *f, void *arg) {
 	IOSCapture *thiz = (IOSCapture*)f->data;
 	if (thiz != NULL) {
-		*(MSVideoSize*)arg = *[thiz getSize];
+		DISPATCH_SYNC_MAIN(^{
+			*(MSVideoSize*)arg = *[thiz getSize];});
 	}
 	return 0;
 }
@@ -724,7 +736,9 @@ static int ioscapture_set_device_orientation (MSFilter *f, void *arg) {
 	if (thiz != NULL) {
 		if (thiz->mDeviceOrientation != *(int*)(arg)) {
 			thiz->mDeviceOrientation = *(int*)(arg);
-			[thiz setSize:thiz->mOutputVideoSize]; //to update size from orientation
+			DISPATCH_SYNC_MAIN(^{
+				[thiz setSize:thiz->mOutputVideoSize]; //to update size from orientation
+			});
 
 			// delete frame if any
 			ms_mutex_lock(&thiz->mutex);
@@ -742,9 +756,10 @@ static int ioscapture_set_device_orientation (MSFilter *f, void *arg) {
 static int ioscapture_set_device_orientation_display (MSFilter *f, void *arg) {
 	IOSCapture *thiz=(IOSCapture*)f->data;
 	if (thiz != NULL) {
-		AVCaptureVideoPreviewLayer *previewLayer = (AVCaptureVideoPreviewLayer *)thiz.layer;
-		if ([previewLayer.connection isVideoOrientationSupported])
-			previewLayer.connection.videoOrientation = Angle2AVCaptureVideoOrientation(*(int*)(arg));
+		DISPATCH_SYNC_MAIN(^{
+			AVCaptureVideoPreviewLayer *previewLayer = (AVCaptureVideoPreviewLayer *)thiz.layer;
+			if ([previewLayer.connection isVideoOrientationSupported])
+				previewLayer.connection.videoOrientation = Angle2AVCaptureVideoOrientation(*(int*)(arg));});
 	}
 	return 0;
 }
@@ -826,7 +841,8 @@ static void ms_v4ios_cam_init(MSWebCam *cam) {
 static MSFilter *ms_v4ios_create_reader(MSWebCam *obj) {
 	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
 	MSFilter *f= ms_factory_create_filter_from_desc(ms_web_cam_get_factory(obj),(&ms_ioscapture_desc));
-	[((IOSCapture*)f->data) openDevice:obj->data];
+	DISPATCH_SYNC_MAIN(^{
+		[((IOSCapture*)f->data) openDevice:obj->data];});
 	[pool drain];
 	return f;
 }
