@@ -17,9 +17,11 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
+
 #include <algorithm>
 #include <memory>
 
+#include <stdint.h>
 #define bool_t matroska_bool_t
 extern "C" {
 #include <matroska/matroska.h>
@@ -1643,7 +1645,7 @@ static matroska_block *matroska_write_block(Matroska *obj, const matroska_frame
 			codecPrivateElt = (ebml_binary *)EBML_MasterAddElt(blockGroup, &MATROSKA_ContextCodecState, FALSE);
 			EBML_BinarySetData(codecPrivateElt, codecPrivateData, codecPrivateDataSize);
 		}
-		block->TrackNumber = trackNum;
+		MATROSKA_BlockSetTrackNum(block, trackNum);
 		MATROSKA_LinkBlockWithReadTracks(block, obj->tracks, TRUE);
 		MATROSKA_LinkBlockWriteSegmentInfo(block, obj->info);
 		MATROSKA_BlockSetKeyframe(block, isKeyFrame);
@@ -2518,6 +2520,9 @@ static ms_bool_t mkv_player_seek_ms(MKVPlayer *obj, int position) {
 	} else {
 		int newpos = obj->reader->seek(position);
 		if (newpos < 0) return FALSE;
+		if (obj->time != newpos) {
+			obj->position_changed = TRUE;
+		}
 		obj->time = newpos;
 	}
 
@@ -2525,7 +2530,6 @@ static ms_bool_t mkv_player_seek_ms(MKVPlayer *obj, int position) {
 		if (obj->players[i]) mkv_track_player_flush(obj->players[i]);
 	}
 
-	obj->position_changed = TRUE;
 	return TRUE;
 }
 
