@@ -108,6 +108,7 @@ typedef struct EncState {
 	MSWorkerThread *process_thread;
 	queue_t entry_q;
 	MSQueue *exit_q;
+	ms_mutex_t vp8_mutex;
 } EncState;
 
 #define MIN_KEY_FRAME_DIST 4 /*since one i-frame is allowed to be 4 times bigger of the target bitrate*/
@@ -134,11 +135,13 @@ static void enc_init(MSFilter *f) {
 #endif
 	s->avpf_enabled = FALSE;
 	enc_reset_frames_state(s);
+	ms_mutex_init(&s->vp8_mutex, NULL);
 	f->data = s;
 }
 
 static void enc_uninit(MSFilter *f) {
 	EncState *s = (EncState *)f->data;
+	ms_mutex_destroy(&s->vp8_mutex);
 	ms_free(s);
 }
 
@@ -152,7 +155,7 @@ static void enc_reset_frames_state(EncState *s){
 	s->frames_state.reconstruct.type=VP8_LAST_FRAME;
 }
 
-static void enc_preprocess(MSFilter *f) {
+static void enc_init_impl(MSFilter *f){
 	EncState *s = (EncState *)f->data;
 	vpx_codec_err_t res;
 	vpx_codec_caps_t caps;
@@ -227,7 +230,12 @@ static void enc_preprocess(MSFilter *f) {
 	} else {
 		vpx_codec_control(&s->codec, VP8E_SET_TOKEN_PARTITIONS, 0);
 	}
+}
 
+static void enc_preprocess(MSFilter *f) {
+	EncState *s = (EncState *)f->data;
+	
+	enc_init_impl(f);
 	s->invalid_frame_reported = FALSE;
 	vp8rtpfmt_packer_init(&s->packer);
 	if (s->avpf_enabled == TRUE) {
@@ -239,7 +247,6 @@ static void enc_preprocess(MSFilter *f) {
 	s->process_thread = ms_worker_thread_new();
 	qinit(&s->entry_q);
 	s->exit_q = ms_queue_new(0, 0, 0, 0);
-
 	s->ready = TRUE;
 }
 
@@ -503,15 +510,17 @@ static void enc_process_frame_task(void *obj) {
 	ms_message("\taltref: count=%" PRIi64 ", picture_id=0x%04x, ack=%s",
 		s->frames_state.altref.count, s->frames_state.altref.picture_id, (s->frames_state.altref.acknowledged == TRUE) ? "Y" : "N");
 #endif
+	ms_mutex_lock(&s->vp8_mutex);
 	err = vpx_codec_encode(&s->codec, &img, s->frame_count, 1, flags, (unsigned long)((double)1000000/(2.0*(double)s->vconf.fps))); /*encoder has half a framerate interval to encode*/
 	if (err) {
+		ms_mutex_unlock(&s->vp8_mutex);
 		ms_error("vpx_codec_encode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec));
 	} else {
 		vpx_codec_iter_t iter = NULL;
 		const vpx_codec_cx_pkt_t *pkt;
 		bctbx_list_t *list = NULL;
 		int current_partition_id = -1;
-
+		
 		/* Update the frames state. */
 		is_ref_frame=FALSE;
 		if (flags & VPX_EFLAG_FORCE_KF) {
@@ -575,6 +584,7 @@ static void enc_process_frame_task(void *obj) {
 				list = bctbx_list_append(list, packet);
 			}
 		}
+		ms_mutex_unlock(&s->vp8_mutex);
 
 #ifdef AVPF_DEBUG
 		ms_message("VP8 encoder picture_id=%i ***| %s | %s | %s | %s", (int)s->picture_id,
@@ -656,6 +666,7 @@ static int enc_set_configuration(MSFilter *f, void *data) {
 	EncState *s = (EncState *)f->data;
 	const MSVideoConfiguration *vconf = (const MSVideoConfiguration *)data;
 	MSVideoSize vsize = s->vconf.vsize;
+	bool_t fps_changed = vconf->fps != s->vconf.fps;
 
 	if (vconf != &s->vconf) memcpy(&s->vconf, vconf, sizeof(MSVideoConfiguration));
 
@@ -668,13 +679,24 @@ static int enc_set_configuration(MSFilter *f, void *data) {
 			ms_warning("Video configuration: cannot change video size when encoder is running, actual=%dx%d, wanted=%dx%d", vsize.width, vsize.height, s->vconf.vsize.width, s->vconf.vsize.height);
 			s->vconf.vsize = vsize;
 		}
-
-		ms_filter_lock(f);
-		vpx_codec_enc_config_set(&s->codec, &s->cfg);
-		ms_filter_unlock(f);
+		else if (fps_changed){
+			/* The VP8 implementation is unreliable for fps change. Simply destroy the encoder and re-create a new one, it is safer.*/
+			if (s->ready){
+				ms_mutex_lock(&s->vp8_mutex);
+				vpx_codec_destroy(&s->codec);
+				enc_init_impl(f);
+				ms_mutex_unlock(&s->vp8_mutex);
+			}
+		}else{
+			ms_mutex_lock(&s->vp8_mutex);
+			if (vpx_codec_enc_config_set(&s->codec, &s->cfg) != 0){
+				ms_error("VP8 encoder new configuration failed to apply.");
+			}
+			ms_mutex_unlock(&s->vp8_mutex);
+		}
 	}
 
-	ms_message("Video configuration set: bitrate=%dbits/s, fps=%f, vsize=%dx%d for encoder [%p]"	, s->vconf.required_bitrate,
+	ms_message("VP8 Video configuration set: bitrate=%dbits/s, fps=%f, vsize=%dx%d for encoder [%p]"	, s->vconf.required_bitrate,
 		   s->vconf.fps, s->vconf.vsize.width, s->vconf.vsize.height, f);
 	return 0;
 }
@@ -886,8 +908,8 @@ typedef struct DecState {
 	MSYuvBufAllocator *allocator;
 	MSPicture outbuf;
 	unsigned int yuv_width, yuv_height;
-	MSQueue q;
 	MSAverageFPS fps;
+	uint32_t last_timestamp;
 	bool_t first_image_decoded;
 	bool_t avpf_enabled;
 	bool_t freeze_on_error;
@@ -913,11 +935,12 @@ static void dec_init(MSFilter *f) {
 	s->yuv_height = 0;
 	s->allocator = ms_yuv_buf_allocator_new();
 	ms_yuv_buf_allocator_set_max_frames(s->allocator, 3);
-	ms_queue_init(&s->q);
 	s->first_image_decoded = FALSE;
 	s->avpf_enabled = FALSE;
 	s->freeze_on_error = TRUE;
 	ms_cond_init(&s->thread_cond, NULL);
+	ms_queue_init(&s->entry_q);
+	ms_queue_init(&s->exit_q);
 	f->data = s;
 	ms_average_fps_init(&s->fps, "VP8 decoder: FPS: %f");
 }
@@ -965,8 +988,7 @@ static void dec_preprocess(MSFilter* f) {
 		s->first_image_decoded = FALSE;
 		s->ready=TRUE;
 	}
-	ms_queue_init(&s->entry_q);
-	ms_queue_init(&s->exit_q);
+	
 	s->thread_running = TRUE;
 	s->waiting = FALSE;
 	ms_thread_create(&s->thread, NULL, dec_processing_thread, f);
@@ -977,7 +999,8 @@ static void dec_uninit(MSFilter *f) {
 	vp8rtpfmt_unpacker_uninit(&s->unpacker);
 	vpx_codec_destroy(&s->codec);
 	ms_yuv_buf_allocator_free(s->allocator);
-	ms_queue_flush(&s->q);
+	ms_queue_flush(&s->entry_q);
+	ms_queue_flush(&s->exit_q);
 	ms_cond_destroy(&s->thread_cond);
 	ms_free(s);
 }
@@ -1009,6 +1032,7 @@ static void *dec_processing_thread(void *obj) {
 			vpx_codec_err_t err;
 			vpx_image_t *img;
 			vpx_codec_iter_t iter = NULL;
+			
 			while ((im = ms_queue_get(&frame)) != NULL) {
 				err = vpx_codec_decode(&s->codec, im->b_rptr, (unsigned int)(im->b_wptr - im->b_rptr), NULL, 0);
 				if ((s->flags & VPX_CODEC_USE_INPUT_FRAGMENTS) && mblk_get_marker_info(im)) {
@@ -1017,6 +1041,7 @@ static void *dec_processing_thread(void *obj) {
 				if (err) {
 					ms_warning("vp8 decode failed : %d %s (%s)\n", err, vpx_codec_err_to_string(err), vpx_codec_error_detail(&s->codec)?vpx_codec_error_detail(&s->codec):"no details");
 				}
+				s->last_timestamp = mblk_get_timestamp_info(im);
 				freemsg(im);
 			}
 	
@@ -1050,7 +1075,7 @@ static void *dec_processing_thread(void *obj) {
 				if (yuv_msg == NULL) {
 					ms_warning("MSVp8Dec: no more output buffers, frame is discarded.");
 				}else{
-					/* scale/copy frame to destination mblk_t */
+					/* copy frame to destination mblk_t */
 					for (i = 0; i < 3; i++) {
 						uint8_t *dest = s->outbuf.planes[i];
 						uint8_t *src = img->planes[i];
@@ -1062,7 +1087,7 @@ static void *dec_processing_thread(void *obj) {
 							src += img->stride[i];
 						}
 					}
-					
+					mblk_set_timestamp_info(yuv_msg, s->last_timestamp);
 					ms_filter_lock(f);
 					ms_queue_put(&s->exit_q, yuv_msg);
 					ms_filter_unlock(f);
@@ -1114,6 +1139,9 @@ static void dec_process(MSFilter *f) {
 
 static void dec_postprocess(MSFilter *f) {
 	DecState *s = (DecState *)f->data;
+	/* Shutdown the decoding thread, but leave entry_q and exit_q as they are.
+	 * In case of immediate restart of the graph, they may contain useful data and it is
+	 * stupid to create a discontinuity because of a graph restart.*/
 	ms_filter_lock(f);
 	s->thread_running = FALSE;
 	if (s->waiting) ms_cond_signal(&s->thread_cond);
