@@ -1,21 +1,21 @@
 /*
-mediastreamer2 library - modular sound and video processing and streaming
-Copyright (C) 2006  Simon MORLAT (simon.morlat@linphone.org)
-
-This program is free software; you can redistribute it and/or
-modify it under the terms of the GNU General Public License
-as published by the Free Software Foundation; either version 2
-of the License, or (at your option) any later version.
-
-This program is distributed in the hope that it will be useful,
-but WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU General Public License for more details.
-
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ * Copyright (c) 2010-2019 Belledonne Communications SARL.
+ *
+ * This file is part of mediastreamer2.
+ *
+ * This program is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program. If not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifdef VIDEO_ENABLED
 
@@ -300,7 +300,11 @@ CPropertyBag::QueryInterface(REFIID riid, void** ppv)
 static int v4w_open_videodevice(V4wState *s, int format, MSVideoSize *vsize)
 {
 	// Initialize COM
+#ifdef ENABLE_MICROSOFT_STORE_APP
+	CoInitializeEx(NULL, COINIT_MULTITHREADED);
+#else
 	CoInitialize(NULL);
+#endif
 
 	// get a Graph
 	HRESULT hr=s->m_pGraph.CoCreateInstance(CLSID_FilterGraph);
@@ -420,7 +424,7 @@ static int v4w_open_videodevice(V4wState *s, int format, MSVideoSize *vsize)
 
 	ULONG nFetched = 0;
 
-	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER, 
+	hr = CoCreateInstance(CLSID_SystemDeviceEnum, NULL, CLSCTX_INPROC_SERVER,
 		IID_ICreateDevEnum, (PVOID *)&pCreateDevEnum);
 	if(FAILED(hr))
 	{
@@ -460,7 +464,7 @@ static int v4w_open_videodevice(V4wState *s, int format, MSVideoSize *vsize)
     CComPtr<IPersistPropertyBag>    pPropertyBag;
 	GetFirstCameraDriver(wzDeviceName);
 
-	hr = s->m_pDeviceFilter.CoCreateInstance( CLSID_VideoCapture ); 
+	hr = s->m_pDeviceFilter.CoCreateInstance( CLSID_VideoCapture );
 	if (FAILED(hr))
 	{
 		return -8;
@@ -517,7 +521,7 @@ IFilterMapper *pMapper = NULL;
 //IEnumMoniker *pEnum = NULL;
 IEnumRegFilters *pEnum = NULL;
 
-hr = CoCreateInstance(CLSID_FilterMapper, 
+hr = CoCreateInstance(CLSID_FilterMapper,
     NULL, CLSCTX_INPROC, IID_IFilterMapper, 
     (void **) &pMapper);
 
