@@ -50,8 +50,6 @@ ms_time(time_t *t) {
 }
 #endif
 
-static void tmmbr_received(const OrtpEventData *evd, void *user_pointer);
-
 
 static void disable_checksums(ortp_socket_t sock) {
 #if defined(DISABLE_CHECKSUMS) && defined(SO_NO_CHECK)
@@ -120,11 +118,22 @@ void media_stream_init(MediaStream *stream, MSFactory *factory, const MSMediaStr
 	if (sessions->dtls_context != NULL) {
 		ms_dtls_srtp_set_stream_sessions(sessions->dtls_context, &stream->sessions);
 	}
+	media_stream_add_tmmbr_handler(stream, media_stream_tmmbr_received, stream);
+}
+
+void media_stream_add_tmmbr_handler(MediaStream *stream, void (*on_tmmbr_received)(const OrtpEventData *evd, void *), void *user_data){
 	ortp_ev_dispatcher_connect(stream->evd
 								, ORTP_EVENT_RTCP_PACKET_RECEIVED
 								, RTCP_RTPFB
-								, (OrtpEvDispatcherCb)tmmbr_received
-								, stream);
+								, (OrtpEvDispatcherCb)on_tmmbr_received
+								, user_data);
+}
+
+void media_stream_remove_tmmbr_handler(MediaStream *stream, void (*on_tmmbr_received)(const OrtpEventData *evd, void *), void *user_data){
+	ortp_ev_dispatcher_disconnect(stream->evd
+							, ORTP_EVENT_RTCP_PACKET_RECEIVED
+							, RTCP_RTPFB
+							, (OrtpEvDispatcherCb)on_tmmbr_received);
 }
 
 RtpSession * ms_create_duplex_rtp_session(const char* local_ip, int loc_rtp_port, int loc_rtcp_port, int mtu) {
@@ -187,6 +196,13 @@ const char * media_stream_type_str(MediaStream *stream) {
 	return ms_format_type_to_string(stream->type);
 }
 
+void media_stream_reset_zrtp_context(MediaStream *stream){
+	MSMediaStreamSessions *sessions = &(stream->sessions);
+	if (sessions && sessions->zrtp_context!=NULL) {
+		ms_zrtp_context_reset(sessions->zrtp_context);
+	}
+}
+
 void ms_media_stream_sessions_uninit(MSMediaStreamSessions *sessions){
 	if (sessions->srtp_context) {
 		ms_srtp_context_delete(sessions->srtp_context);
@@ -211,7 +227,7 @@ void ms_media_stream_sessions_uninit(MSMediaStreamSessions *sessions){
 }
 
 void media_stream_free(MediaStream *stream) {
-	ortp_ev_dispatcher_disconnect(stream->evd, ORTP_EVENT_RTCP_PACKET_RECEIVED, RTCP_RTPFB, (OrtpEvDispatcherCb)tmmbr_received);
+	media_stream_remove_tmmbr_handler(stream, media_stream_tmmbr_received, stream);
 	if (stream->sessions.zrtp_context != NULL) {
 		ms_zrtp_set_stream_sessions(stream->sessions.zrtp_context, NULL);
 	}
@@ -273,29 +289,46 @@ void media_stream_enable_adaptive_jittcomp(MediaStream *stream, bool_t enabled)
 	rtp_session_enable_adaptive_jitter_compensation(stream->sessions.rtp_session, enabled);
 }
 
+
+/* This function decides whether it is necessary to send dummy stun packets for firewall opening. */
+static void media_stream_configure_stun_packet_sending(MediaStream *stream){
+	bool_t stun_enabled = TRUE;
+	if (stream->ice_check_list) stun_enabled = FALSE;
+	if (stream->sessions.rtp_session->bundle && !stream->sessions.rtp_session->is_primary){
+		stun_enabled = FALSE;
+	}
+	if (stream->rtpsend != NULL) {
+		ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_ENABLE_STUN, &stun_enabled);
+		if (stream->sessions.dtls_context) {
+			/* In case STUN packets are necessary, and DTLS is used and encryption mandatory is set to TRUE,
+			 * no packets will be emitted at all. We must configure the rtpsend filter to regularly send dummy stun packets
+			 * to ensure that firewall gets open to the remote endpoint. 
+			 * Note that we are unable to check here if encryption mandatory is on (due to order of operation that might be
+			 * undetermined), but it is acceptable to send dummy stun packets even if encryption mandatory is off.
+			 */
+			ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_ENABLE_STUN_FORCED, &stun_enabled);
+		}
+	}
+}
+
+
 void media_stream_enable_dtls(MediaStream *stream, const MSDtlsSrtpParams *params){
 	if (stream->sessions.dtls_context == NULL) {
 		MSDtlsSrtpParams params_copy = *params;
 		ms_message("Start DTLS media stream context in stream session [%p]", &(stream->sessions));
 		if (params_copy.mtu == 0) params_copy.mtu = ms_factory_get_mtu(stream->factory);
+		
 		stream->sessions.dtls_context=ms_dtls_srtp_context_new(&(stream->sessions), &params_copy);
+		media_stream_configure_stun_packet_sending(stream);
 	}
 }
 
 void media_stream_set_ice_check_list(MediaStream *stream, IceCheckList *cl) {
-	bool_t stun_enabled = TRUE;
 	stream->ice_check_list = cl;
 	if (stream->ice_check_list != NULL) {
 		ice_check_list_set_rtp_session(stream->ice_check_list, stream->sessions.rtp_session);
-		stun_enabled = FALSE;
-	}
-	if (stream->sessions.rtp_session->bundle && !stream->sessions.rtp_session->is_primary){
-		ms_message("Disabling stun sending for rtp session %p", stream->sessions.rtp_session);
-		stun_enabled = FALSE;
-	}
-	if (stream->rtpsend != NULL) {
-		ms_filter_call_method(stream->rtpsend, MS_RTP_SEND_ENABLE_STUN, &stun_enabled);
 	}
+	media_stream_configure_stun_packet_sending(stream);
 }
 
 bool_t media_stream_dtls_supported(void){
@@ -545,29 +578,48 @@ RtpSession * media_stream_get_rtp_session(const MediaStream *stream) {
 MSCryptoSuite ms_crypto_suite_build_from_name_params(const MSCryptoSuiteNameParams *descrption){
 	const char *name=descrption->name, *parameters=descrption->params;
 	if (keywordcmp ( "AES_CM_128_HMAC_SHA1_80",name ) == 0 ){
-		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) return MS_NO_CIPHER_SHA1_80;
-		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) return MS_AES_128_NO_AUTH;
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) return MS_AES_128_SHA1_80_NO_CIPHER;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) return MS_AES_128_SHA1_80_SRTP_NO_CIPHER;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) return MS_AES_128_SHA1_80_SRTCP_NO_CIPHER;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) return MS_AES_128_SHA1_80_NO_AUTH;
 		else return MS_AES_128_SHA1_80;
 	}else if ( keywordcmp ( "AES_CM_128_HMAC_SHA1_32",name ) == 0 ){
-		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
-		if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) return MS_AES_128_NO_AUTH;
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) return MS_AES_128_SHA1_32_NO_AUTH;
 		else return MS_AES_128_SHA1_32;
 	}else if ( keywordcmp ("AES_256_CM_HMAC_SHA1_32", name) == 0 ){
-		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
-		if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
-		return MS_AES_256_SHA1_32;
+
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
+		else return MS_AES_256_SHA1_32;
 	}else if ( keywordcmp ("AES_256_CM_HMAC_SHA1_80", name) == 0 ){
-		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
-		if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
-		return MS_AES_256_SHA1_80;
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
+		else return MS_AES_256_SHA1_80;
 	}else if ( keywordcmp ("AES_CM_256_HMAC_SHA1_80", name) == 0 ){
-        if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
-        if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
-        return MS_AES_CM_256_SHA1_80;
-    }
+		if (parameters && strstr(parameters,"UNENCRYPTED_SRTP") && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTP")) goto error;
+		else if (parameters && strstr(parameters,"UNENCRYPTED_SRTCP")) goto error;
+		else if (parameters && strstr(parameters,"UNAUTHENTICATED_SRTP")) goto error;
+		else return MS_AES_CM_256_SHA1_80;
+	}
 error:
-    ms_error("Unsupported crypto suite '%s' with parameters '%s'",name, parameters ? parameters : "");
-    return MS_CRYPTO_SUITE_INVALID;
+	ms_error("Unsupported crypto suite '%s' with parameters '%s'",name, parameters ? parameters : "");
+	return MS_CRYPTO_SUITE_INVALID;
+}
+
+bool_t ms_crypto_suite_is_unencrypted(MSCryptoSuite cs){
+	return (cs == MS_AES_128_SHA1_80_SRTP_NO_CIPHER) || (cs == MS_AES_128_SHA1_80_SRTCP_NO_CIPHER) || (cs == MS_AES_128_SHA1_80_NO_CIPHER);
+}
+
+bool_t ms_crypto_suite_is_unauthenticated(MSCryptoSuite cs){
+	return (cs == MS_AES_128_SHA1_80_NO_AUTH) || (cs == MS_AES_128_SHA1_32_NO_AUTH);
 }
 
 int ms_crypto_suite_to_name_params(MSCryptoSuite cs, MSCryptoSuiteNameParams *params ){
@@ -582,11 +634,23 @@ int ms_crypto_suite_to_name_params(MSCryptoSuite cs, MSCryptoSuiteNameParams *pa
      	case MS_AES_128_SHA1_32:
      		params->name="AES_CM_128_HMAC_SHA1_32";
      		break;
-     	case MS_AES_128_NO_AUTH:
+     	case MS_AES_128_SHA1_80_NO_AUTH:
      		params->name="AES_CM_128_HMAC_SHA1_80";
      		params->params="UNAUTHENTICATED_SRTP";
      		break;
-     	case MS_NO_CIPHER_SHA1_80:
+     	case MS_AES_128_SHA1_32_NO_AUTH:
+     		params->name="AES_CM_128_HMAC_SHA1_32";
+     		params->params="UNAUTHENTICATED_SRTP";
+     		break;
+     	case MS_AES_128_SHA1_80_SRTP_NO_CIPHER:
+     		params->name="AES_CM_128_HMAC_SHA1_80";
+     		params->params="UNENCRYPTED_SRTP";
+     		break;
+     	case MS_AES_128_SHA1_80_SRTCP_NO_CIPHER:
+     		params->name="AES_CM_128_HMAC_SHA1_80";
+     		params->params="UNENCRYPTED_SRTCP";
+     		break;
+     	case MS_AES_128_SHA1_80_NO_CIPHER:
      		params->name="AES_CM_128_HMAC_SHA1_80";
      		params->params="UNENCRYPTED_SRTP UNENCRYPTED_SRTCP";
      		break;
@@ -622,6 +686,8 @@ const char *ms_resource_type_to_string(MSResourceType type){
 			return "MSResourceRtp";
 		case MSResourceSoundcard:
 			return "MSResourceSoundcard";
+		case MSResourceVoid:
+			return "MSResourceVoid";
 	}
 	return "INVALID";
 }
@@ -644,6 +710,8 @@ bool_t ms_media_resource_is_consistent(const MSMediaResource *r){
 		case MSResourceInvalid:
 			ms_error("Invalid resource type specified");
 			return FALSE;
+		case MSResourceVoid:
+			return TRUE;
 	}
 	ms_error("Unsupported media resource type [%i]", (int)r->type);
 	return FALSE;
@@ -698,53 +766,55 @@ static int update_bitrate_limit_from_tmmbr(MediaStream *obj, int br_limit){
 	return br_limit;
 }
 
-static void tmmbr_received(const OrtpEventData *evd, void *user_pointer) {
+void media_stream_process_tmmbr(MediaStream *ms, int tmmbr_mxtbr){
+	ms_message("MediaStream[%p]: received a TMMBR for bitrate %i kbits/s"
+				, ms, (int)(tmmbr_mxtbr/1000));
+	tmmbr_mxtbr = update_bitrate_limit_from_tmmbr(ms, tmmbr_mxtbr);
+	if (tmmbr_mxtbr < 0) return;
+
+#ifdef VIDEO_ENABLED
+	if (ms->type == MSVideo) {
+		const char* preset = video_stream_get_video_preset((VideoStream*) ms);
+
+		if (preset && strcmp(preset, "custom") == 0) {
+			MSVideoConfiguration *vconf_list = NULL;
+			MSVideoConfiguration current_vconf, vconf;
+			int new_bitrate_limit;
+
+			ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, &vconf_list);
+
+			if (vconf_list != NULL) {
+				ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION, &current_vconf);
+
+				vconf = ms_video_find_best_configuration_for_size_and_bitrate(vconf_list, current_vconf.vsize, ms_factory_get_cpu_count(ms->factory), tmmbr_mxtbr);
+
+				new_bitrate_limit = tmmbr_mxtbr < vconf.bitrate_limit ? tmmbr_mxtbr : vconf.bitrate_limit;
+				ms_message("Changing video encoder's output bitrate to %i", new_bitrate_limit);
+				current_vconf.required_bitrate = new_bitrate_limit;
+
+				if (ms_filter_call_method(ms->encoder,MS_VIDEO_ENCODER_SET_CONFIGURATION, &current_vconf) != 0) {
+					ms_warning("Failed to apply fps and bitrate constraint to %s", ms->encoder->desc->name);
+				}
+			}
+
+			return;
+		}
+
+		if (!ms->video_quality_controller) {
+			ms->video_quality_controller = ms_video_quality_controller_new((VideoStream*) ms);
+		}
+
+		ms_video_quality_controller_update_from_tmmbr(ms->video_quality_controller, tmmbr_mxtbr);
+	}
+#endif
+}
+
+void media_stream_tmmbr_received(const OrtpEventData *evd, void *user_pointer) {
 	MediaStream *ms = (MediaStream *)user_pointer;
 	switch (rtcp_RTPFB_get_type(evd->packet)) {
 		case RTCP_RTPFB_TMMBR: {
 			int tmmbr_mxtbr = (int)rtcp_RTPFB_tmmbr_get_max_bitrate(evd->packet);
-
-			ms_message("MediaStream[%p]: received a TMMBR for bitrate %i kbits/s"
-						, ms, (int)(tmmbr_mxtbr/1000));
-			tmmbr_mxtbr = update_bitrate_limit_from_tmmbr(ms, tmmbr_mxtbr);
-			if (tmmbr_mxtbr < 0) return;
-
-
-#ifdef VIDEO_ENABLED
-			if (ms->type == MSVideo) {
-				const char* preset = video_stream_get_video_preset((VideoStream*) ms);
-
-				if (preset && strcmp(preset, "custom") == 0) {
-					MSVideoConfiguration *vconf_list = NULL;
-					MSVideoConfiguration current_vconf, vconf;
-					int new_bitrate_limit;
-
-					ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION_LIST, &vconf_list);
-
-					if (vconf_list != NULL) {
-						ms_filter_call_method(ms->encoder, MS_VIDEO_ENCODER_GET_CONFIGURATION, &current_vconf);
-
-						vconf = ms_video_find_best_configuration_for_size_and_bitrate(vconf_list, current_vconf.vsize, ms_factory_get_cpu_count(ms->factory), tmmbr_mxtbr);
-
-						new_bitrate_limit = tmmbr_mxtbr < vconf.bitrate_limit ? tmmbr_mxtbr : vconf.bitrate_limit;
-						ms_message("Changing video encoder's output bitrate to %i", new_bitrate_limit);
-						current_vconf.required_bitrate = new_bitrate_limit;
-
-						if (ms_filter_call_method(ms->encoder,MS_VIDEO_ENCODER_SET_CONFIGURATION, &current_vconf) != 0) {
-							ms_warning("Failed to apply fps and bitrate constraint to %s", ms->encoder->desc->name);
-						}
-					}
-
-					break;
-				}
-
-				if (!ms->video_quality_controller) {
-					ms->video_quality_controller = ms_video_quality_controller_new((VideoStream*) ms);
-				}
-
-				ms_video_quality_controller_update_from_tmmbr(ms->video_quality_controller, tmmbr_mxtbr);
-			}
-#endif
+			media_stream_process_tmmbr(ms, tmmbr_mxtbr);
 			break;
 		}
 		default:
