@@ -57,6 +57,12 @@ static const FormatDesc _format_desc_list[] = {
 static bool_t four_cc_compare(const FourCC arg1, const FourCC arg2);
 static MSFileFormat four_cc_to_file_format(const FourCC four_cc);
 
+enum GraphState {
+	Uninit = 0,
+	ResourcesAcquired,
+	GraphReady
+};
+
 struct _MSMediaPlayer {
 	MSFactory *factory;
 	MSFilter *player;
@@ -70,6 +76,7 @@ struct _MSMediaPlayer {
 	MSTicker *ticker;
 	MSFileFormat format;
 	bool_t is_open;
+	enum GraphState graph_state;
 	int loop_interval;
 	char *filename;
 	MSMediaPlayerEofCallback eof_cb;
@@ -164,8 +171,14 @@ void ms_media_player_set_window_id(MSMediaPlayer *obj, void *window_id) {
 
 bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath) {
 	wave_header_t header;
-	int fd;
+	bctbx_vfs_file_t *fp;
 	char *tmp;
+
+	if(obj->is_open) {
+		ms_error("Player is already opened, close it first!");
+		return FALSE;
+	}
+
 	ms_message("Opening %s", filepath);
 	if(access(filepath, F_OK) != 0) {
 		ms_error("Cannot open %s. File does not exist", filepath);
@@ -177,16 +190,16 @@ bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath) {
 	}
 	switch(obj->format) {
 	case MS_FILE_FORMAT_WAVE:
-		fd = open(filepath, O_RDONLY);
-		if(fd == -1) {
+		fp = bctbx_file_open2(bctbx_vfs_get_default(), filepath, O_RDONLY);
+		if(fp == NULL) {
 			ms_error("Cannot open %s", filepath);
 			return FALSE;
 		}
-		if(ms_read_wav_header_from_fd(&header, fd) == -1) {
+		if(ms_read_wav_header_from_fp(&header, fp) == -1) {
 			ms_error("Cannot open %s. Invalid WAV format", filepath);
 			return FALSE;
 		}
-		close(fd);
+		bctbx_file_close(fp);
 		if(wave_header_get_format_type(&header) != WAVE_FORMAT_PCM) {
 			ms_error("Cannot open %s. Codec not supported", filepath);
 			return FALSE;
@@ -211,23 +224,10 @@ bool_t ms_media_player_open(MSMediaPlayer *obj, const char *filepath) {
 		return FALSE;
 	}
 	ms_free(tmp);
-	_create_decoders(obj);
-	_create_sinks(obj);
-	if(!_link_all(obj)) {
-		ms_error("Cannot open %s. Could not build playing graph", filepath);
-		_destroy_graph(obj);
-		return FALSE;
-	}
-	ms_filter_add_notify_callback(obj->player, _eof_filter_notify_cb, obj, FALSE);
-	ms_filter_call_method(obj->player, MS_PLAYER_SET_LOOP, &obj->loop_interval);
-	if (obj->snd_card) {
-		ms_snd_card_notify_audio_session_activated(obj->snd_card, TRUE);
-	}
-	obj->ticker = ms_ticker_new();
-	ms_ticker_set_name(obj->ticker, "Player");
-	ms_ticker_attach(obj->ticker, obj->player);
+	
 	obj->is_open = TRUE;
 	obj->filename = ms_strdup(filepath);
+
 	return TRUE;
 }
 
@@ -235,24 +235,69 @@ bool_t ms_media_player_get_is_video_available(MSMediaPlayer *obj) {
 	return obj->video_sink != NULL;
 }
 
+void ms_media_player_set_volume_gain(MSMediaPlayer *player, float gain) {
+	if (player->audio_sink) {
+		ms_filter_call_method(player->audio_sink, MS_AUDIO_PLAYBACK_SET_VOLUME_GAIN, &gain);
+	}
+}
+
+float ms_media_player_get_volume_gain(const MSMediaPlayer *player) {
+	float gain = 0.0f;
+	if (player->audio_sink) {
+		ms_filter_call_method(player->audio_sink, MS_AUDIO_PLAYBACK_GET_VOLUME_GAIN, &gain);
+	}
+	return gain;
+}
+
+MSFilter *ms_media_player_get_video_sink(const MSMediaPlayer *const obj) {
+	return obj->video_sink;
+}
+
 void ms_media_player_close(MSMediaPlayer *obj) {
 	if(obj->is_open) {
+		
 		ms_message("MSMediaPlayer: closing file.");
-		ms_ticker_detach(obj->ticker, obj->player);
-		ms_ticker_destroy(obj->ticker);
 		ms_filter_call_method_noarg(obj->player, MS_PLAYER_CLOSE);
-		_unlink_all(obj);
-		_destroy_graph(obj);
 		obj->is_open = FALSE;
+		if (obj->graph_state == GraphReady){
+			ms_media_player_stop(obj);
+		}
 		ms_free(obj->filename); obj->filename = NULL;
 	}
 }
 
+void ms_media_player_prepare(MSMediaPlayer *obj) {
+	if (obj->graph_state < ResourcesAcquired) {
+		ms_snd_card_set_stream_type(obj->snd_card, MS_SND_CARD_STREAM_MEDIA);
+		_create_decoders(obj);
+		_create_sinks(obj);
+		obj->graph_state = ResourcesAcquired;
+	}
+}
+
 bool_t ms_media_player_start(MSMediaPlayer *obj) {
 	if(!obj->is_open) {
 		ms_error("Cannot start playing. No file has been opened");
 		return FALSE;
 	}
+	if (obj->graph_state < GraphReady){
+		ms_media_player_prepare(obj);
+		if(!_link_all(obj)) {
+			ms_error("Could not build playing graph");
+			_destroy_graph(obj);
+			return FALSE;
+		}
+		ms_filter_add_notify_callback(obj->player, _eof_filter_notify_cb, obj, FALSE);
+		ms_filter_call_method(obj->player, MS_PLAYER_SET_LOOP, &obj->loop_interval);
+		if (obj->snd_card) {
+			ms_snd_card_notify_audio_session_activated(obj->snd_card, TRUE);
+		}
+		obj->ticker = ms_ticker_new();
+		ms_ticker_set_name(obj->ticker, "Player");
+		ms_ticker_attach(obj->ticker, obj->player);
+		obj->graph_state = GraphReady;
+	}
+	
 	if(ms_filter_call_method_noarg(obj->player, MS_PLAYER_START) == -1) {
 		ms_error("Could not play %s. Playing filter failed to start", obj->filename);
 		return FALSE;
@@ -262,11 +307,19 @@ bool_t ms_media_player_start(MSMediaPlayer *obj) {
 
 void ms_media_player_stop(MSMediaPlayer *obj) {
 	int seek_pos = 0;
-	if(obj->is_open) {
-		ms_message("MSMediaPlayer: sotping playback.");
+	if(obj->is_open && obj->player) {
+		ms_message("MSMediaPlayer: stopping playback.");
 		ms_filter_call_method_noarg(obj->player, MS_PLAYER_PAUSE);
 		ms_filter_call_method(obj->player, MS_PLAYER_SEEK_MS, &seek_pos);
 	}
+	if (obj->graph_state == GraphReady){
+		ms_ticker_detach(obj->ticker, obj->player);
+		ms_ticker_destroy(obj->ticker);
+		
+		_unlink_all(obj);
+		_destroy_graph(obj);
+		obj->graph_state = Uninit;
+	}
 }
 
 void ms_media_player_pause(MSMediaPlayer *obj) {
@@ -351,31 +404,21 @@ MSFileFormat ms_media_player_get_file_format(const MSMediaPlayer *obj) {
 /* Private functions */
 static bool_t _get_format(const char *filepath, MSFileFormat *format) {
 	FourCC four_cc;
-	size_t data_read;
-	FILE *file = fopen(filepath, "rb");
-	if(file == NULL) {
+	bctbx_vfs_file_t *fp = bctbx_file_open(bctbx_vfs_get_default(), filepath, "rb");
+	if(fp == NULL) {
 		ms_error("Could not open %s: %s", filepath, strerror(errno));
 		goto err;
 	}
-	data_read = fread(four_cc, 4, 1, file);
-	if (data_read == 0) {
-		const char *error_msg;
-		if (ferror(file)) {
-			error_msg = strerror(errno);
-		} else if (feof(file)) {
-			error_msg = "end of file reached";
-		} else {
-			error_msg = "unknown error";
-		}
-		ms_error("Could not read the FourCC of %s: %s", filepath, error_msg);
+	if (bctbx_file_read2(fp, four_cc, sizeof(FourCC)) == BCTBX_VFS_ERROR){
+		ms_error("Could not read the FourCC of %s: %s",filepath,strerror(errno));
 		goto err;
 	}
 	*format = four_cc_to_file_format(four_cc);
-	fclose(file);
+	bctbx_file_close(fp);
 	return TRUE;
 
 err:
-	if (file) fclose(file);
+	if (fp) bctbx_file_close(fp);
 	*format = MS_FILE_FORMAT_UNKNOWN;
 	return FALSE;
 }
@@ -404,6 +447,11 @@ static void _create_decoders(MSMediaPlayer *obj) {
 				nchannels = obj->audio_pin_fmt.fmt->nchannels;
 				ms_filter_call_method(obj->audio_decoder, MS_FILTER_SET_SAMPLE_RATE, &sample_rate);
 				ms_filter_call_method(obj->audio_decoder, MS_FILTER_SET_NCHANNELS, &nchannels);
+				if (strcmp(obj->audio_pin_fmt.fmt->encoding, "opus") == 0) {
+					// Disable PLC in Opus decoder to prevent white noise when pausing media player
+					const char *fmtp = "plc=0";
+					ms_filter_call_method(obj->audio_decoder, MS_FILTER_ADD_FMTP, (void *)fmtp);
+				}
 			}
 		}
 		if(obj->video_pin_fmt.fmt) {
