@@ -24,7 +24,6 @@
 #include "mediastreamer2/msfileplayer.h"
 #include "private.h"
 
-
 static void ring_player_event_handler(void *ud, MSFilter *f, unsigned int evid, void *arg){
 	RingStream *stream=(RingStream*)ud;
 
@@ -60,7 +59,9 @@ RingStream * ring_start_with_cb(MSFactory* factory, const char *file, int interv
 	MSPinFormat pinfmt={0};
 
 	stream=(RingStream *)ms_new0(RingStream,1);
-	stream->card = sndcard;
+	if (sndcard != NULL) {
+		stream->card = ms_snd_card_ref(sndcard);
+	}
 	if (file) {
 		stream->source=_ms_create_av_player(file,factory);
 		if (stream->source == NULL){
@@ -76,7 +77,7 @@ RingStream * ring_start_with_cb(MSFactory* factory, const char *file, int interv
 	if (func!=NULL)
 		ms_filter_add_notify_callback(stream->source,func,user_data,FALSE);
 	stream->gendtmf=ms_factory_create_filter(factory, MS_DTMF_GEN_ID);
-	stream->sndwrite=ms_snd_card_create_writer(sndcard);
+	stream->sndwrite= (sndcard != NULL) ? ms_snd_card_create_writer(sndcard) : ms_factory_create_filter(factory, MS_VOID_SINK_ID);
 	stream->write_resampler=ms_factory_create_filter(factory, MS_RESAMPLE_ID);
 
 	if (file){
@@ -99,8 +100,8 @@ RingStream * ring_start_with_cb(MSFactory* factory, const char *file, int interv
 			pinfmt.fmt = ms_factory_get_audio_format(factory, "pcm", 8000, 1, NULL);
 		}
 	}
-	srcrate = pinfmt.fmt->rate;
-	srcchannels = pinfmt.fmt->nchannels;
+	dstrate = srcrate = pinfmt.fmt->rate;
+	dstchannels = srcchannels = pinfmt.fmt->nchannels;
 
 	ms_filter_call_method(stream->sndwrite,MS_FILTER_SET_SAMPLE_RATE,&srcrate);
 	ms_filter_call_method(stream->sndwrite,MS_FILTER_GET_SAMPLE_RATE,&dstrate);
@@ -154,10 +155,12 @@ void ring_play_dtmf(RingStream *stream, char dtmf, int duration_ms){
 }
 
 void ring_stop_dtmf(RingStream *stream){
+//	ms_message("DADA [RingStream] stop dtmf ringing card is %s", ((stream->card) ? stream->card->id : "No default"));
 	ms_filter_call_method_noarg(stream->gendtmf, MS_DTMF_GEN_STOP);
 }
 
 void ring_stop(RingStream *stream){
+//	ms_message("DADA [RingStream] stop ringing card in stream is %s", ((stream->card) ? stream->card->id : "No card"));
 	MSConnectionHelper h;
 
 	if (stream->ticker){
@@ -180,5 +183,37 @@ void ring_stop(RingStream *stream){
 	if (stream->decoder) ms_filter_destroy(stream->decoder);
 	if (stream->write_resampler)
 		ms_filter_destroy(stream->write_resampler);
+	if (stream->card) ms_snd_card_unref(stream->card);
 	ms_free(stream);
+
+}
+
+/*
+ * note: Only AAudio and OpenSLES leverage internal ID for output streams.
+ */
+static void ring_stream_configure_output_snd_card(RingStream *stream) {
+	MSSndCard *card = stream->card;
+	if (stream->sndwrite) {
+		if(ms_filter_implements_interface(stream->sndwrite, MSFilterAudioPlaybackInterface)) {
+			ms_filter_call_method(stream->sndwrite, MS_AUDIO_PLAYBACK_SET_INTERNAL_ID, card);
+			ms_message("[RingStream] set output sound card for %s:%p to %s", ms_filter_get_name(stream->sndwrite), stream->sndwrite, card->id);
+		}
+	}
+}
+
+void ring_stream_set_output_ms_snd_card(RingStream *stream, MSSndCard * sndcard_playback) {
+	if (stream->card) {
+		ms_snd_card_unref(stream->card);
+		stream->card = NULL;
+	}
+	stream->card = ms_snd_card_ref(sndcard_playback);
+	ring_stream_configure_output_snd_card(stream);
+}
+
+MSSndCard * ring_stream_get_output_ms_snd_card(RingStream *stream) {
+	// If stream is null, then do not try to access the card
+	if (stream)
+		return stream->card;
+
+	return NULL;
 }
