@@ -1846,6 +1846,10 @@ static void configure_video_preview_source(VideoPreview *stream) {
 	if (ms_filter_has_method(stream->source, MS_VIDEO_DISPLAY_SET_DEVICE_ORIENTATION)) {
 		ms_filter_call_method(stream->source, MS_VIDEO_DISPLAY_SET_DEVICE_ORIENTATION, &stream->device_orientation);
 	}
+	
+	ms_filter_add_notify_callback(stream->source, event_cb, stream, FALSE);
+	/* It is important that the internal_event_cb is called synchronously! */
+	ms_filter_add_notify_callback(stream->source, internal_event_cb, stream, TRUE);
 
 	if (!ms_filter_implements_interface(stream->source, MSFilterVideoEncoderInterface)) {
 		ms_filter_call_method(stream->source, MS_FILTER_SET_VIDEO_SIZE, &vsize);
@@ -2246,3 +2250,15 @@ void video_stream_close_remote_record(VideoStream *stream){
 		ms_filter_call_method_noarg(recorder, MS_RECORDER_CLOSE);
 	}
 }
+
+void video_stream_enable_fec(VideoStream *stream, char* local_ip, int local_port, int local_rtcp_port, char* remote_ip, int remote_port, int L, int D){
+    ms_factory_set_mtu(stream->ms.factory, ms_factory_get_mtu(stream->ms.factory) - (12 + L*4));
+    RtpSession *fec_session = ms_create_duplex_rtp_session(ms_is_ipv6(local_ip) ? "::" : "0.0.0.0", local_port+10, local_rtcp_port+10, 0);
+    rtp_session_set_remote_addr(fec_session, remote_ip, remote_port+10);
+    fec_session->fec_stream = NULL;
+    JBParameters jitter_params;
+    rtp_session_get_jitter_buffer_params(stream->ms.sessions.rtp_session, &jitter_params);
+    const FecParameters *fec_params = fec_params_new(L, D, jitter_params.nom_size);
+    FecStream *fec_stream = fec_stream_new(stream->ms.sessions.rtp_session, fec_session, fec_params);
+    stream->ms.sessions.rtp_session->fec_stream = fec_stream;
+}
